# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import urllib

import lxml.html
import ox

DEFAULT_MAX_RESULTS = 10
DEFAULT_TIMEOUT = 24*60*60

def read_url(url, data=None, headers=ox.net.DEFAULT_HEADERS, timeout=DEFAULT_TIMEOUT):
    return ox.cache.read_url(url, data, headers, timeout, unicode=True)

def quote_plus(s):
    if not isinstance(s, bytes):
        s = s.encode('utf-8')
    return urllib.parse.quote_plus(s)

def find(query, max_results=DEFAULT_MAX_RESULTS, timeout=DEFAULT_TIMEOUT):
    """
    Return max_results tuples with title, url, description 

    >>> find("The Matrix site:imdb.com", 1)[0][0]
    'The Matrix (1999) - IMDb'

    >>> find("The Matrix site:imdb.com", 1)[0][1]
    'http://www.imdb.com/title/tt0133093/'
    """
    results = []
    url = 'https://eu1.startpage.com/do/search?nosteeraway=1&abp=1&language=english&cmd=process_search&query=%s&x=0&y=0&cat=web&engine0=v1all' % quote_plus(query)

    data = read_url(url, timeout=timeout)
    doc = lxml.html.document_fromstring(data)
    for r in doc.xpath("//div[contains(@class, 'result')]"):
        t = r.find('h3')
        if t is not None:
            title = t.text_content().strip()
            url = t.find('a').attrib['href']
            description = r.find_class('desc')[0].text_content()
            results.append((title, url, description))
            if len(results) >= max_results:
                break
    return results




from ox.cache import read_url
from ox import find_re, strip_tags
import re

base = 'http://www.lookupbyisbn.com'

def get_data(isbn):
    r = {}
    url = '%s/Search/Book/%s/1' % (base, isbn)

    data = read_url(url).decode('utf-8')
    m = re.compile('href="(/Lookup/Book/[^"]+?)"').findall(data)
    if m:
        ids = m[0].split('/')
        r['isbn'] = ids[-2]
        r['asin'] = ids[-3]
        url = '%s%s' % (base, m[0])
        data = read_url(url).decode('utf-8')
        r["title"] = find_re(data, "<h2>(.*?)</h2>")
        keys = {
            'author': 'Author(s)',
            'publisher': 'Publisher',
            'date': 'Publication date',
            'edition': 'Edition',
            'binding': 'Binding',
            'volume': 'Volume(s)',
            'pages': 'Pages',
        }
        for key in keys:
            r[key] = find_re(data, '<span class="title">%s:</span>(.*?)</li>'% re.escape(keys[key]))
            if r[key] == '--':
                r[key] = ''
            if key == 'pages' and r[key]:
                r[key] = int(r[key])
        desc = find_re(data, '<h2>Description:<\/h2>(.*?)<div ')
        desc = desc.replace('<br /><br />', ' ').replace('<br /> ', ' ').replace('<br />', ' ')
        r['description'] = strip_tags(desc).strip()
        if r['description'] == u'Description of this item is not available at this time.':
            r['description'] = ''
        r['cover'] = find_re(data, '<img src="(.*?)" alt="Book cover').replace('._SL160_', '')
    return r




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import re
from datetime import datetime
from urllib.parse import quote

import lxml.html
import ox
from ox.cache import read_url

def find(query=None, user=None, timeout=60):
    if user:
        url = 'https://twitter.com/' + quote(user)
    else:
        url = 'https://twitter.com/search/' + quote(query)
    data = ox.cache.read_url(url, timeout=timeout).decode('utf-8')
    doc = lxml.html.document_fromstring(data)
    tweets = []
    for e in doc.xpath("//div[contains(@class, 'original-tweet')]"):
        t = lxml.html.tostring(e)
        text = e.xpath(".//p[contains(@class, 'js-tweet-text')]")[0]
        html = lxml.html.tostring(text, encoding='unicode').strip()
        text = ox.decode_html(ox.strip_tags(html)).strip()
        user = re.compile('data-name="(.*?)"').findall(t)[0]
        user = ox.decode_html(ox.strip_tags(user)).strip()
        tweets.append({
            'id': re.compile('data-tweet-id="(\d+)"').findall(t)[0],
            'user-id': re.compile('data-user-id="(\d+)"').findall(t)[0],
            'name': re.compile('data-screen-name="(.*?)"').findall(t)[0],
            'time': datetime.fromtimestamp(int(re.compile('data-time="(\d+)"').findall(t)[0])),
            'user': user,
            'text': text,
            'html': html,
        })
    return tweets



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import json

from ox.cache import read_url
from ox import find_re

class Freebase(dict):
    def __init__(self, id, timeout=-1):
        url = "http://ids.freebaseapps.com/get_ids?id=/authority/imdb/title/tt%s" % id
        '''
            "http://graph.freebase.com/imdb.title.tt%s" % id
            might also be of interest at some point, right now not much info
        '''
        data = read_url(url, unicode=True)
        try:
            data = json.loads(data)
        except ValueError:
            return
        '''
        for key in data:
            self[key] = data[key]
        '''
        for key in ('id', 'guid', 'name'):
            self[key] = data[key]
        keys = {
            'wikipedia': '/wikipedia/en',
            'netflix': '/authority/netflix/movie',
            'nytimes': '/source/nytimes/movie',
            'metacritic': '/source/metacritic/movie',
        }
        for key in keys:
            links = filter(lambda x: x['namespace'] == keys[key],data['ids'])
            if links:
                self[key] = links[0]['uri']

        if 'nytimes' in self:
            self['nytimes'] = self['nytimes'].replace('_/overview', '%s/overview' % self['name'].replace(' ', '-'))
            self['amgId'] = find_re(self['nytimes'], 'movie/(\d+)/')






# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
# GPL 2008
import re
import string
from html.entities import name2codepoint

letters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'

# Configuration for add_links() function

LEADING_PUNCTUATION = ['(', '<', '&lt;']
TRAILING_PUNCTUATION = ['.', ',', ')', '>', '\n', '&gt;', "'", '"']

# list of possible strings used for bullets in bulleted lists
DOTS = ['&middot;', '*', '\xe2\x80\xa2', '&#149;', '&bull;', '&#8226;']

unencoded_ampersands_re = re.compile(r'&(?!(\w+|#\d+);)')
word_split_re = re.compile(r'(\s+)')
punctuation_re = re.compile('^(?P<lead>(?:%s)*)(?P<middle>.*?)(?P<trail>(?:%s)*)$' % (
                            '|'.join([re.escape(x) for x in LEADING_PUNCTUATION]),
                            '|'.join([re.escape(x) for x in TRAILING_PUNCTUATION])))
simple_email_re = re.compile(r'^\S+@[a-zA-Z0-9._-]+\.[a-zA-Z0-9._-]+$')
link_target_attribute_re = re.compile(r'(<a [^>]*?)target=[^\s>]+')
html_gunk_re = re.compile(r'(?:<br clear="all">|<i><\/i>|<b><\/b>|<em><\/em>|<strong><\/strong>|<\/?smallcaps>|<\/?uppercase>)', re.IGNORECASE)
hard_coded_bullets_re = re.compile(r'((?:<p>(?:%s).*?[a-zA-Z].*?</p>\s*)+)' % '|'.join([re.escape(x) for x in DOTS]), re.DOTALL)
trailing_empty_content_re = re.compile(r'(?:<p>(?:&nbsp;|\s|<br \/>)*?</p>\s*)+\Z')


def escape(html):
    '''
    Returns the given HTML with ampersands, quotes and carets encoded

    >>> escape('html "test" & <brothers>')
    'html &quot;test&quot; &amp; &lt;brothers&gt;'
    '''
    if not isinstance(html, str):
        html = str(html)
    return html.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;').replace('"', '&quot;').replace("'", '&apos;')

def linebreaks(value):
    '''
    Converts newlines into <p> and <br />
    '''
    value = re.sub(r'\r\n|\r|\n', '\n', value)  # normalize newlines
    paras = re.split('\n{2,}', value)
    paras = ['<p>%s</p>' % p.strip().replace('\n', '<br />') for p in paras]
    return '\n\n'.join(paras)

def strip_tags(value):
    """
    Returns the given HTML with all tags stripped
    
    >>> strip_tags('some <h2>title</h2> <script>asdfasdf</script>')
    'some title asdfasdf'
    """
    return re.sub(r'<[^>]*?>', '', value)

stripTags = strip_tags

def strip_spaces_between_tags(value):
    "Returns the given HTML with spaces between tags normalized to a single space"
    return re.sub(r'>\s+<', '> <', value)

def strip_entities(value):
    "Returns the given HTML with all entities (&something;) stripped"
    return re.sub(r'&(?:\w+|#\d);', '', value)

def fix_ampersands(value):
    "Returns the given HTML with all unencoded ampersands encoded correctly"
    return unencoded_ampersands_re.sub('&amp;', value)

def add_links(text, trim_url_limit=None, nofollow=False):
    """
    Converts any URLs in text into clickable links. Works on http://, https:// and
    www. links. Links can have trailing punctuation (periods, commas, close-parens)
    and leading punctuation (opening parens) and it'll still do the right thing.

    If trim_url_limit is not None, the URLs in link text will be limited to
    trim_url_limit characters.

    If nofollow is True, the URLs in link text will get a rel="nofollow" attribute.
    """
    trim_url = lambda x, limit=trim_url_limit: limit is not None and (x[:limit] + (len(x) >= limit and '...' or ''))  or x
    words = word_split_re.split(text)
    nofollow_attr = nofollow and ' rel="nofollow"' or ''
    for i, word in enumerate(words):
        match = punctuation_re.match(word)
        if match:
            lead, middle, trail = match.groups()
            if middle.startswith('www.') or ('@' not in middle and not middle.startswith('http://') and
                                             len(middle) > 0 and middle[0] in letters + string.digits and
                                             (middle.endswith('.org') or
                                              middle.endswith('.net') or
                                              middle.endswith('.com'))):
                middle = '<a href="http://%s"%s>%s</a>' % (middle, nofollow_attr, trim_url(middle))
            if middle.startswith('http://') or middle.startswith('https://'):
                middle = '<a href="%s"%s>%s</a>' % (middle, nofollow_attr, trim_url(middle))
            if '@' in middle and not middle.startswith('www.') and ':' not in middle \
                    and simple_email_re.match(middle):
                middle = '<a href="mailto:%s">%s</a>' % (middle, middle)
            if lead + middle + trail != word:
                words[i] = lead + middle + trail
    return ''.join(words)

urlize = add_links

def clean_html(text):
    """
    Cleans the given HTML. Specifically, it does the following:
        * Converts <b> and <i> to <strong> and <em>.
        * Encodes all ampersands correctly.
        * Removes all "target" attributes from <a> tags.
        * Removes extraneous HTML, such as presentational tags that open and
          immediately close and <br clear="all">.
        * Converts hard-coded bullets into HTML unordered lists.
        * Removes stuff like "<p>&nbsp;&nbsp;</p>", but only if it's at the
          bottom of the text.
    """
    from .text import normalize_newlines
    text = normalize_newlines(text)
    text = re.sub(r'<(/?)\s*b\s*>', '<\\1strong>', text)
    text = re.sub(r'<(/?)\s*i\s*>', '<\\1em>', text)
    text = fix_ampersands(text)
    # Remove all target="" attributes from <a> tags.
    text = link_target_attribute_re.sub('\\1', text)
    # Trim stupid HTML such as <br clear="all">.
    text = html_gunk_re.sub('', text)
    # Convert hard-coded bullets into HTML unordered lists.

    def replace_p_tags(match):
        s = match.group().replace('</p>', '</li>')
        for d in DOTS:
            s = s.replace('<p>%s' % d, '<li>')
        return '<ul>\n%s\n</ul>' % s
    text = hard_coded_bullets_re.sub(replace_p_tags, text)
    # Remove stuff like "<p>&nbsp;&nbsp;</p>", but only if it's at the bottom of the text.
    text = trailing_empty_content_re.sub('', text)
    return text

# This pattern matches a character entity reference (a decimal numeric
# references, a hexadecimal numeric reference, or a named reference).
charrefpat = re.compile(r'&(#(\d+|x[\da-fA-F]+)|[\w.:-]+);?')

def decode_html(html):
    """
    >>> decode_html('me &amp; you and &#36;&#38;%')
    'me & you and $&%'
    >>> decode_html('&#x80;')
    '\u20ac'
    >>> decode_html('Anniversary of Daoud&apos;s Republic')
    "Anniversary of Daoud's Republic"
    """
    if isinstance(html, bytes):
        html = html.decode('utf-8')
    uchr = chr

    def entitydecode(match, uchr=uchr):
        entity = match.group(1)
        if entity == '#x80':
            return '€'
        elif entity.startswith('#x'):
            return uchr(int(entity[2:], 16))
        elif entity.startswith('#'):
            return uchr(int(entity[1:]))
        elif entity in name2codepoint:
            return uchr(name2codepoint[entity])
        elif entity == 'apos':
            return "'"
        else:
            return match.group(0)
    return charrefpat.sub(entitydecode, html).replace('\xa0', ' ')

def highlight(text, query, hlClass="hl"):
    """
    >>> highlight('me &amp; you and &#36;&#38;%', 'and')
    'me &amp; you <span class="hl">and</span> &#36;&#38;%'
    """
    if query:
        text = text.replace('<br />', '|')
        query = re.escape(query).replace('\ ', '.')
        m = re.compile("(%s)" % query, re.IGNORECASE).findall(text)
        for i in m:
            text = re.sub("(%s)" % re.escape(i).replace('\ ', '.'), '<span class="%s">\\1</span>' % hlClass, text)
        text = text.replace('|', '<br />')
    return text

def escape_html(value):
    '''
    >>> escape_html('<script> foo')
    '&lt;script&gt; foo'
    >>> escape_html('&lt;script&gt; foo')
    '&lt;script&gt; foo'
    '''
    return escape(decode_html(value))

def sanitize_html(html, tags=None, global_attributes=[]):
    '''
    >>> sanitize_html('http://foo.com, bar')
    '<a href="http://foo.com">http://foo.com</a>, bar'
    >>> sanitize_html('http://foo.com/foobar?foo, bar')
    '<a href="http://foo.com/foobar?foo">http://foo.com/foobar?foo</a>, bar'
    >>> sanitize_html('(see: www.foo.com)')
    '(see: <a href="http://www.foo.com">www.foo.com</a>)'
    >>> sanitize_html('foo@bar.com')
    '<a href="mailto:foo@bar.com">foo@bar.com</a>'
    >>> sanitize_html(sanitize_html('foo@bar.com'))
    '<a href="mailto:foo@bar.com">foo@bar.com</a>'
    >>> sanitize_html('<a href="http://foo.com" onmouseover="alert()">foo</a>')
    '<a href="http://foo.com">foo</a>'
    >>> sanitize_html('<a href="javascript:alert()">foo</a>')
    '&lt;a href="javascript:alert()"&gt;foo&lt;/a&gt;'
    >>> sanitize_html('[http://foo.com foo]')
    '<a href="http://foo.com">foo</a>'
    >>> sanitize_html('<div style="direction: rtl">foo</div>')
    '<div style="direction: rtl">foo</div>'
    >>> sanitize_html('<script>alert()</script>')
    '&lt;script&gt;alert()&lt;/script&gt;'
    >>> sanitize_html("'foo' < 'bar' && \\"foo\\" > \\"bar\\"")
    '\\'foo\\' &lt; \\'bar\\' &amp;&amp; "foo" &gt; "bar"'
    >>> sanitize_html('<b>foo')
    '<b>foo</b>'
    >>> sanitize_html('<b>foo</b></b>')
    '<b>foo</b>'
    >>> sanitize_html('Anniversary of Daoud&apos;s Republic')
    "Anniversary of Daoud's Republic"
    >>> sanitize_html('')
    ''
    >>> sanitize_html(' ')
    ' '
    >>> sanitize_html('&nbsp;')  # canonicalised to a space: okay, I suppose
    ' '
    >>> sanitize_html('\u00a0')  # also nbsp
    ' '
    '''
    if not tags:
        valid_url = '^((https?:\/\/|\/|mailto:).*?)'
        tags = [
            # inline formatting
            {'name': 'b'},
            {'name': 'bdi'},
            {'name': 'code'},
            {'name': 'em'},
            {'name': 'i'},
            {'name': 'q'},
            {'name': 's'},
            {'name': 'span'},
            {'name': 'strong'},
            {'name': 'sub'},
            {'name': 'sup'},
            {'name': 'u'},
            # block formatting
            {'name': 'blockquote'},
            {'name': 'cite'},
            {
                'name': 'div',
                'optional': ['style'],
                'validation': {
                    'style': '^direction: rtl$'
                }
            },
            {'name': 'h1'},
            {'name': 'h2'},
            {'name': 'h3'},
            {'name': 'h4'},
            {'name': 'h5'},
            {'name': 'h6'},
            {'name': 'p'},
            {'name': 'pre'},
            # lists
            {'name': 'li'},
            {'name': 'ol'},
            {'name': 'ul'},
            # definition lists
            {'name': 'dl'},
            {'name': 'dt'},
            {'name': 'dd'},
            # tables
            {'name': 'table'},
            {'name': 'tbody'},
            {'name': 'td'},
            {'name': 'tfoot'},
            {'name': 'th'},
            {'name': 'thead'},
            {'name': 'tr'},
            # other
            {'name': '[]'},
            {
                'name': 'a',
                'required': ['href'],
                'optional': ['target'],
                'validation': {
                    'href': valid_url,
                    'target': '^_blank$',
                }
            },
            {'name': 'br'},
            {
                'name': 'iframe',
                'optional': ['width', 'height'],
                'required': ['src'],
                'validation': {
                    'width': '^\d+$',
                    'height': '^\d+$',
                    'src': valid_url
                }
            },
            {
                'name': 'img',
                'optional': ['width', 'height'],
                'required': ['src'],
                'validation': {
                    'width': '^\d+$',
                    'height': '^\d+$',
                    'src': valid_url
                },
            },
            {'name': 'figure'},
            {'name': 'figcaption'}
        ]

    tag_re = re.compile('<(/)?([^\ /]+)(.*?)(/)?>')
    attr_re = re.compile('([^=\ ]+)="([^"]+)"')

    escaped = {}
    level = 0
    non_closing_tags = ['img', 'br']
    required_attributes = {}
    validation = {}
    valid_attributes = {}
    valid_tags = set([tag['name'] for tag in tags if tag['name'] != '[]'])

    for tag in tags:
        valid_attributes[tag['name']] = tag.get('required', []) \
            + tag.get('optional', []) + global_attributes
        required_attributes[tag['name']] = tag.get('required', [])
        validation[tag['name']] = tag.get('validation', {})

    if '[]' in validation:
        html = re.sub(
            re.compile('\[((https?:\/\/|\/).+?) (.+?)\]', re.IGNORECASE),
            '<a href="\\1">\\3</a>', html)

    parts = split_tags(html)
    for i, part in enumerate(parts):
        is_tag = i % 2
        if is_tag:
            t = tag_re.findall(part)
            if not t:
                parts[i] = escape_html(decode_html(part))
                continue
            closing, name, attributes, end = t[0]
            closing = closing != ''
            a = attr_re.findall(attributes)
            attrs = dict(a)

            if not closing and name not in non_closing_tags:
                level += 1

            if not attrs and attributes or name not in valid_tags:
                valid = False
            else:
                valid = True
                for key in set(attrs) - set(valid_attributes[name]):
                    del attrs[key]
                for key in required_attributes[tag['name']]:
                    if key not in attrs:
                        valid = False

            if valid:
                for attr in attrs:
                    if attr in validation[name]:
                        if not re.compile(validation[name][attr]).findall(attrs[attr]):
                            valid = False
                            break

            if valid and closing:
                valid = not escaped.get(level)
            else:
                escaped[level] = not valid
            if closing:
                level -= 1
            if valid:
                parts[i] = '<%s%s%s>' % (
                    ('/' if closing else ''),
                    name,
                    (' ' + ' '.join(['%s="%s"' % (key, attrs[key]) for key, value in a if key in attrs])
                        if not closing and attrs else '')
                )
            else:
                parts[i] = escape_html(decode_html(part))
        else:
            parts[i] = escape_html(decode_html(part))
    html = ''.join(parts)
    html = html.replace('\n\n', '<br/><br/>')
    html = add_links(html)
    return sanitize_fragment(html)

def split_tags(string):
    tags = []

    def collect(match):
        tags.append(match.group(0))
        return '\0'
    strings = re.sub('<[^<>]+>', collect, string).split('\0')
    tags.append('')
    return [item for sublist in zip(strings, tags) for item in sublist][:-1]

def sanitize_fragment(html):
    '''
    Ensures that tags are closed (or not, as appropriate), attributes
    are quoted, etc. Does not strip potentially-malicious HTML: use
    sanitize_html() for that.

    >>> sanitize_fragment('<span lang="en">')
    '<span lang="en"></span>'
    >>> sanitize_fragment('<span lang=en></span>')
    '<span lang="en"></span>'
    >>> sanitize_fragment('<br><br/></br>')
    '<br><br>'
    >>> sanitize_fragment('<a href="javascript:alert()">foo</a>')
    '<a href="javascript:alert()">foo</a>'
    >>> sanitize_fragment('')
    ''
    >>> sanitize_fragment(' ')
    ' '
    >>> sanitize_fragment('&nbsp;')
    '\\xa0'
    >>> sanitize_fragment('\\u00a0')  # nbsp
    '\\xa0'
    >>> sanitize_fragment('\\ufeff')  # zero-width no-break space
    '\\ufeff'
    '''

    '''
    #html5lib reorders arguments, so not usable
    import html5lib
    return html5lib.parseFragment(html).toxml().decode('utf-8')
    '''
    if not html.strip():
        return html
    import lxml.html
    try:
        body = lxml.html.document_fromstring(html).find('body')
    except lxml.etree.ParserError as e:
        if e.args and e.args[0] == 'Document is empty':
            return html
        raise e
    html = lxml.html.tostring(body, encoding='utf-8')[6:-7].decode('utf-8')
    if html.startswith('<p>') and html.endswith('</p>'):
        html = html[3:-4]
    return html



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import math
import re
import string

def toAZ(num):
    """
    Converts an integer to bijective base 26 string using A-Z

    >>> for i in range(1, 1000): assert fromAZ(toAZ(i)) == i

    >>> toAZ(1)
    'A'

    >>> toAZ(4461)
    'FOO'

    >>> toAZ(1234567890)
    'CYWOQVJ'
    """
    if num < 1:
        raise ValueError("must supply a positive integer")
    digits = string.ascii_uppercase
    az = ''
    while num != 0:
        num, r = divmod(num, 26)
        u, r = divmod(r - 1, 26)
        num += u
        az = digits[r] + az
    return az

encode_base26 = toAZ

def fromAZ(num):
    """
    Converts a bijective base 26 string to an integer

    >>> fromAZ('A')
    1
    >>> fromAZ('AA')
    27
    >>> fromAZ('AAA')
    703
    >>> fromAZ('FOO')
    4461
    """
    num = num.replace('-', '')
    digits = string.ascii_uppercase
    r = 0
    for exp, char in enumerate(reversed(num)):
        r = r + (pow(26, exp) * (digits.index(char) + 1))
    return r

def to26(q):
    """
    Converts an integer to base 26

    >>> for i in range(0, 1000): assert from26(to26(i)) == i
    
    >>> to26(0)
    'A'

    >>> to26(347485647)
    'BDGKMAP'
    """
    if q < 0:
        raise ValueError("must supply a positive integer")
    base26 = string.ascii_uppercase
    converted = []
    while q != 0:
        q, r = divmod(q, 26)
        l = base26[r]
        converted.insert(0, l)
    return "".join(converted) or 'A'

decode_base26 = toAZ

def from26(q):
    """
    Converts an base 26 string to an integer
    >>> from26('A')
    0
    """
    base26 = string.ascii_uppercase
    q = q.replace('-', '')
    r = 0
    for i in q:
        r = r * 26 + base26.index(i.upper())
    return r

def to32(q):
    """
    Converts an integer to base 32
    We exclude 4 of the 26 letters: I L O U.
    http://www.crockford.com/wrmg/base32.html

    >>> for i in range(0, 1000): assert from32(to32(i)) == i
    
    >>> to32(0)
    '0'

    >>> to32(347485647)
    'ABCDEF'

    >>> to32(555306645)
    'GHJKMN'

    >>> to32(800197332334559)
    'PQRSTVWXYZ'

    >>> to32(32)
    '10'

    >>> to32(119292)
    '3MFW'

    >>> to32(939387374)
    'VZVTFE'

    >>> to32(-1)
    Traceback (most recent call last):
        ...
    ValueError: must supply a positive integer
    """

    if q < 0:
        raise ValueError("must supply a positive integer")
    letters = "0123456789ABCDEFGHJKMNPQRSTVWXYZ"
    converted = []
    while q != 0:
        q, r = divmod(q, 32)
        l = letters[r]
        converted.insert(0, l)
    return "".join(converted) or '0'

def from32(q):
    """
    Converts an base 32 string to an integer
    We exclude 4 of the 26 letters: I L O U.
    http://www.crockford.com/wrmg/base32.html

    >>> from32('A')
    10

    >>> from32('i')
    1

    >>> from32('Li1l')
    33825

    >>> from32('10')
    32
    """
    _32map = {
        '0': 0,
        'O': 0,
        '1': 1,
        'I': 1,
        'L': 1,
        '2': 2,
        '3': 3,
        '4': 4,
        '5': 5,
        '6': 6,
        '7': 7,
        '8': 8,
        '9': 9,
        'A': 10,
        'B': 11,
        'C': 12,
        'D': 13,
        'E': 14,
        'F': 15,
        'G': 16,
        'H': 17,
        'J': 18,
        'K': 19,
        'M': 20,
        'N': 21,
        'P': 22,
        'Q': 23,
        'R': 24,
        'S': 25,
        'T': 26,
        'V': 27,
        'W': 28,
        'X': 29,
        'Y': 30,
        'Z': 31,
    }
    base32 = ('0123456789' + string.ascii_uppercase)[:32]
    q = q.replace('-', '')
    q = ''.join([base32[_32map[i.upper()]] for i in q])
    return int(q, 32)

def to36(q):
    """
    Converts an integer to base 36 (a useful scheme for human-sayable IDs
    like 'fuck' (739172), 'shit' (1329077) or 'hitler' (1059538851)).

    >>> to36(35)
    'z'
    >>> to36(119292)
    '2k1o'
    >>> int(to36(939387374), 36)
    939387374
    >>> to36(0)
    '0'
    >>> to36(-393)
    Traceback (most recent call last):
        ...
    ValueError: must supply a positive integer
    """
    if q < 0:
        raise ValueError("must supply a positive integer")
    letters = "0123456789abcdefghijklmnopqrstuvwxyz"
    converted = []
    while q != 0:
        q, r = divmod(q, 36)
        converted.insert(0, letters[r])
    return "".join(converted) or '0'

def from36(q):
    return int(q, 36)

def int_value(strValue, default=''):
    """
    >>> int_value('abc23')
    '23'

    >>> int_value(' abc23')
    '23'

    >>> int_value('ab')
    ''
    """
    try:
        val = re.compile('(\d+)').findall(str(strValue).strip())[0]
    except:
        val = default
    return val

def float_value(strValue, default=''):
    """
    >>> float_value('abc23.4')
    '23.4'

    >>> float_value(' abc23.4')
    '23.4'

    >>> float_value('ab')
    ''
    """
    try:
        val = re.compile('([\d.]+)').findall(str(strValue).strip())[0]
    except:
        val = default
    return val

def format_number(number, longName, shortName):
    """
    Return the number in a human-readable format (23 KB, 23.4 MB, 23.42 GB)
    
    >>> format_number(123, 'Byte', 'B')
    '123 Bytes'

    >>> format_number(1234, 'Byte', 'B')
    '1 KB'

    >>> format_number(1234567, 'Byte', 'B')
    '1.2 MB'

    >>> format_number(1234567890, 'Byte', 'B')
    '1.15 GB'

    >>> format_number(1234567890123456789, 'Byte', 'B')
    '1,096.5166 PB'

    >>> format_number(-1234567890123456789, 'Byte', 'B')
    '-1,096.5166 PB'

    """
    if abs(number) < 1024:
        return '%s %s%s' % (format_thousands(number), longName, number != 1 and 's' or '')
    prefix = ['K', 'M', 'G', 'T', 'P']
    for i in range(5):
        if abs(number) < math.pow(1024, i + 2) or i == 4:
            n = number / math.pow(1024, i + 1)
            return '%s %s%s' % (format_thousands('%.*f' % (i, n)), prefix[i], shortName)

def format_thousands(number, separator=','):
    """
    Return the number with separators (1,000,000)
    
    >>> format_thousands(1)
    '1'
    >>> format_thousands(1000)
    '1,000'
    >>> format_thousands(1000000)
    '1,000,000'
    """
    string = str(number).split('.')
    l = []
    for i, character in enumerate(reversed(string[0])):
        if i and (not (i % 3)):
            l.insert(0, separator)
        l.insert(0, character)
    string[0] = ''.join(l)
    return '.'.join(string)

def format_bits(number):
    return format_number(number, 'bit', 'b')

def format_bytes(number):
    return format_number(number, 'byte', 'B')

def format_pixels(number):
    return format_number(number, 'pixel', 'px')

def format_currency(amount, currency="$"):
    if amount:
        temp = "%.2f" % amount
        profile = re.compile(r"(\d)(\d\d\d[.,])")
        while 1:
            temp, count = re.subn(profile, r"\1,\2", temp)
            if not count:
                break
        if temp.startswith('-'):
            return "-" + currency + temp[1:-3]
        return currency + temp[:-3]
    else:
        return ""

def plural(amount, unit, plural='s'):
    '''
    >>> plural(1, 'unit')
    '1 unit'
    >>> plural(2, 'unit')
    '2 units'
    '''
    if abs(amount) != 1:
        if plural == 's':
            unit = unit + plural
        else:
            unit = plural
    return "%s %s" % (format_thousands(amount), unit)

def format_duration(ms, verbosity=0, years=True, hours=True, milliseconds=True):
    '''
    verbosity
        0: D:HH:MM:SS
        1: Dd Hh Mm Ss
        2: D days H hours M minutes S seconds
    years
        True: 366 days are 1 year 1 day
        False: 366 days are 366 days
    hours
        True: 30 seconds are 00:00:30
        False: 30 seconds are 00:30
    milliseconds
        True: always display milliseconds
        False: never display milliseconds
    >>> format_duration(1000 * 60 * 60 * 24 * 366)
    '1:001:00:00:00.000'
    >>> format_duration(1000 * 60 * 60 * 24 * 366, years=False)
    '366:00:00:00.000'
    >>> format_duration(1000 * 60 * 60 * 24 * 365 + 2003, verbosity=2)
    '1 year 2 seconds 3 milliseconds'
    >>> format_duration(1000 * 30, hours=False, milliseconds=False)
    '00:30'
    '''
    if not ms and ms != 0:
        return ''
    if years:
        y = int(ms / 31536000000)
        d = int(ms % 31536000000 / 86400000)
    else:
        d = int(ms / 86400000)
    h = int(ms % 86400000 / 3600000)
    m = int(ms % 3600000 / 60000)
    s = int(ms % 60000 / 1000)
    ms = ms % 1000
    if verbosity == 0:
        if years and y:
            duration = "%d:%03d:%02d:%02d:%02d" % (y, d, h, m, s)
        elif d:
            duration = "%d:%02d:%02d:%02d" % (d, h, m, s)
        elif hours or h:
            duration = "%02d:%02d:%02d" % (h, m, s)
        else:
            duration = "%02d:%02d" % (m, s)
        if milliseconds:
            duration += ".%03d" % ms
    else:
        if verbosity == 1:
            durations = ["%sd" % d, "%sh" % h, "%sm" % m, "%ss" % s]
            if years:
                durations.insert(0, "%sy" % y)
            if milliseconds:
                durations.append("%sms" % ms)
        else:
            durations = [plural(d, 'day'), plural(h, 'hour'),
                         plural(m, 'minute'), plural(s, 'second')]
            if years:
                durations.insert(0, plural(y, 'year'))
            if milliseconds:
                durations.append(plural(ms, 'millisecond'))
        durations = filter(lambda x: not x.startswith('0'), durations)
        duration = ' '.join(durations)
    return duration

def format_timecode(seconds):
    '''
    >>> format_timecode(3599.999)
    '00:59:59.999'
    '''
    seconds = float(seconds)
    d = int(seconds / 86400)
    h = int(seconds % 86400 / 3600)
    m = int(seconds % 3600 / 60)
    s = float(seconds % 60)
    duration = "%s%02d:%02d:%06.3f" % ('%d:' % d if d else '', h, m, s)
    return duration

def parse_timecode(string):
    '''
    Takes a formatted timecode, returns seconds

    >> parse_timecode('1:02:03:04.05')
    93784.05
    >> parse_timecode('3')
    3.0
    >> parse_timecode('2:')
    120
    >> parse_timecode('1::')
    3600.0
    '''
    timecode = 0
    for i, v in enumerate(list(reversed(string.split(':')))[:4]):
        timecode += float(v) * (86400 if i == 3 else pow(60, i))
    return timecode

def ms2runtime(ms, shortenLong=False):
    # deprecated - use format_duration
    '''
    >>> ms2runtime(5000)
    '5 seconds'
    >>> ms2runtime(500000)
    '8 minutes 20 seconds'
    >>> ms2runtime(50000000)
    '13 hours 53 minutes 20 seconds'
    >>> ms2runtime(50000000-20000)
    '13 hours 53 minutes'
    '''
    if shortenLong and ms > 1000 * 60 * 60 * 24 * 464:
        return format_duration(ms, verbosity=1, milliseconds=False)
    return format_duration(ms, verbosity=2, milliseconds=False)

def ms2playtime(ms, hours=False):
    # deprecated - use format_duration
    '''
    >>> ms2playtime(5000)
    '00:05'
    >>> ms2playtime(500000)
    '08:20'
    >>> ms2playtime(50000000)
    '13:53:20'
    '''
    return format_duration(ms, hours=False, years=False, milliseconds=False)

def ms2time(ms):
    # deprecated - use format_duration
    '''
    >>> ms2time(44592123)
    '12:23:12.123'
    '''
    return format_duration(ms, years=False)

def time2ms(timeString):
    '''
    >>> time2ms('12:23:12.123')
    44592123
    '''
    ms = 0.0
    p = timeString.split(':')
    for i in range(len(p)):
        _p = p[i]
        if _p.endswith('.'):
            _p = _p[:-1]
        ms = ms * 60 + float(_p)
    return int(ms * 1000)

def shift_time(offset, timeString):
    newTime = time2ms(timeString) + offset
    return ms2time(newTime)




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
# GPL 2007-2012

from threading import Event
from hashlib import sha1
import os

from .bencode3 import bencode, bdecode

__all__ = ['create_torrent', 'get_info_hash', 'get_torrent_info', 'get_files', 'get_torrent_size']

def create_torrent(file, url, params={}, flag=Event(),
                   progress=lambda x: None, progress_percent=1):
    "Creates a torrent for a given file, using url as tracker url"
    from .makemetafile import make_meta_file
    return make_meta_file(file, url, params, flag, progress, progress_percent)

def get_info_hash(torrentFile):
    "Returns Torrent Info Hash from torrent file"
    metainfo_file = open(torrentFile, 'rb')
    metainfo = bdecode(metainfo_file.read())
    info = metainfo['info']
    return sha1(bencode(info)).hexdigest()

def get_torrent_info(data=None, file=None):
    if file:
        if not isinstance(file, bytes):
            file = file.encode('utf-8')
        with open(file, 'rb') as f:
            data = f.read()

    "Returns Torrent Info from torrent file"
    tinfo = {}
    metainfo = bdecode(data)
    info = metainfo['info']
    piece_length = info['piece length']
    if 'length' in info:
        # let's assume we just have one file
        file_length = info['length']
    else:
        # let's assume we have a directory structure
        file_length = 0;
        for f in info['files']:
            file_length += f['length']
    for key in info:
        if key != 'pieces':
            tinfo[key] = info[key]
    for key in metainfo:
        if key != 'info':
            tinfo[key] = metainfo[key]
    tinfo['size'] = file_length
    tinfo['hash'] = sha1(bencode(info)).hexdigest()
    if 'announce' in metainfo:
        tinfo['announce'] = metainfo['announce']
    if file:
        tinfo['timestamp'] = os.stat(file).st_ctime
    return tinfo

def get_files(data):
    files = []
    info = get_torrent_info(data=data)
    if 'files' in info:
        for f in info['files']:
            path = [info['name'], ]
            path.extend(f['path'])
            files.append(os.path.join(*path))
    else:
        files.append(info['name'])
    return files

def get_torrent_size(file):
    "Returns Size of files in torrent file in bytes"
    return get_torrent_info(file=file)['size']




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import re
import json
from multiprocessing.pool import ThreadPool

from ..cache import read_url
from .. import decode_html
from ..utils import datetime


def cleanup(key, data, data_type):
    if data:
        if isinstance(data[0], str):
            #FIXME: some types need strip_tags
            #data = [strip_tags(decode_html(p)).strip() for p in data]
            data = [decode_html(p).strip() for p in data]
        elif isinstance(data[0], list) or isinstance(data[0], tuple):
            data = [cleanup(key, p, data_type) for p in data]
        while len(data) == 1 and not isinstance(data, str):
            data = data[0]
        if data_type == 'list' and isinstance(data, str):
            data = [data, ]
    elif data_type != 'list':
        data = ''
    return data

class SiteParser(dict):
    baseUrl = ''
    regex = {}
    pool = ThreadPool(8)
    debug = False

    def get_url(self, page):
        return "%s%s" % (self.baseUrl, page)

    def read_url(self, url, timeout):
        if url not in self._cache:
            self._cache[url] = read_url(url, timeout=timeout, unicode=True)
        return self._cache[url]

    def __init__(self, timeout=-1):
        self._cache = {}
        urls = list(set(self.get_url(self.regex[key]['page']) for key in self.regex))
        self.pool.map(self.get_url, urls)

        for key in self.regex:
            url = self.get_url(self.regex[key]['page'])
            data = self.read_url(url, timeout)
            if isinstance(self.regex[key]['re'], str):
                data = re.compile(self.regex[key]['re'], re.DOTALL).findall(data)
                data = cleanup(key, data, self.regex[key]['type'])
            elif callable(self.regex[key]['re']):
                data = self.regex[key]['re'](data)
            else:
                for r in self.regex[key]['re']:
                    if callable(r):
                        f = r
                    else:
                        f = re.compile(r, re.DOTALL).findall
                    if isinstance(data, str):
                        data = f(data)
                    else:
                        data = [f(d) for d in data]
                        data = cleanup(key, data, self.regex[key]['type'])
            def apply_f(f, data):
                if data and isinstance(data[0], list):
                    data = [f(d) for d in data]
                else:
                    data = f(data)
                return data
            if self.regex[key]['type'] == 'float' and data:
                data = apply_f(float, data)
            elif self.regex[key]['type'] == 'int' and data:
                data = apply_f(int, data)
            elif self.regex[key]['type'] == 'date':
                parse_date = lambda d: d and datetime.strptime('-'.join(d), '%m-%d-%Y').strftime('%Y-%m-%d')
                data = apply_f(parse_date, data)
            elif self.regex[key]['type'] == 'json':
                if isinstance(data, list) and len(data) == 1:
                    data = data[0]
                    data = json.loads(data)
            if data:
                self[key] = data




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import division, print_function
import codecs
import re

import chardet
import ox


__all__ = []


def _detect_encoding(fp):
    bomDict = {  # bytepattern : name
        (0x00, 0x00, 0xFE, 0xFF): "utf_32_be",
        (0xFF, 0xFE, 0x00, 0x00): "utf_32_le",
        (0xFE, 0xFF, None, None): "utf_16_be",
        (0xFF, 0xFE, None, None): "utf_16_le",
        (0xEF, 0xBB, 0xBF, None): "utf_8",
    }

    # go to beginning of file and get the first 4 bytes
    oldFP = fp.tell()
    fp.seek(0)
    (byte1, byte2, byte3, byte4) = fp.read(4)

    # try bom detection using 4 bytes, 3 bytes, or 2 bytes
    bomDetection = bomDict.get((byte1, byte2, byte3, byte4))
    if not bomDetection:
        bomDetection = bomDict.get((byte1, byte2, byte3, None))
        if not bomDetection:
            bomDetection = bomDict.get((byte1, byte2, None, None))
    # if BOM detected, we're done :-)
    fp.seek(oldFP)
    if bomDetection:
        return bomDetection
    encoding = 'latin-1'
    # more character detecting magick using http://chardet.feedparser.org/
    fp.seek(0)
    rawdata = fp.read()
    # if data can be decoded as utf-8 use that, try chardet otherwise
    # chardet detects utf-8 as ISO-8859-2 most of the time
    try:
        rawdata.decode('utf-8')
        encoding = 'utf-8'
    except:
        encoding = chardet.detect(rawdata)['encoding']
    fp.seek(oldFP)
    return encoding


def load(filename, offset=0):
    '''Parses an srt file

    filename: path to an srt file
    offset (float, seconds): shift all in/out points by offset

    Returns list with dicts that have in, out, value and id
    '''
    srt = []
    with open(filename, 'rb') as f:
        encoding = _detect_encoding(f)
        data = f.read()
    try:
        data = data.decode(encoding)
    except:
        try:
            data = data.decode('latin-1')
        except:
            print("failed to detect encoding, giving up")
            return []
    return loads(data, offset)

def loads(data, offset=0):
    '''Parses an srt file

    filename: path to an srt file
    offset (float, seconds): shift all in/out points by offset

    Returns list with dicts that have in, out, value and id
    '''
    srt = []

    def parse_time(t):
        return offset + ox.time2ms(t.replace(',', '.')) / 1000

    data = data.replace('\r\n', '\n')
    if not data.endswith('\n\n'):
        data += '\n\n'
    regexp = r'(\d\d:\d\d:\d\d[,.]\d\d\d)\s*?-->\s*?(\d\d:\d\d:\d\d[,.]\d\d\d).*?\n(.*?)\n\n'
    srts = re.compile(regexp, re.DOTALL)
    i = 0
    for s in srts.findall(data):
        _s = {
            'id': str(i),
            'in': parse_time(s[0]),
            'out': parse_time(s[1]),
            'value': s[2].strip()
        }
        srt.append(_s)
        i += 1
    return srt


def _srt_timecode(t):
    return ox.format_duration(t * 1000, years=False).replace('.', ',')


def encode(data):
    """Encodes subtitles into SRT format

    data: list of dicts with 'in', 'out': float and 'value': unicode

    Returns: a UTF-8-encoded bytestring

    >>> encode([{'in': 1.25, 'out': 60 * 60 + 1, 'value': u'touch\\u00E9'}])
    '\\xef\\xbb\\xbf1\\r\\n00:00:01,250 --> 01:00:01,000\\r\\ntouch\\xc3\\xa9\\r\\n\\r\\n'
    """

    srt = u''

    for i, s in enumerate(data, 1):
        srt += '%d\r\n%s --> %s\r\n%s\r\n\r\n' % (
            i,
            _srt_timecode(s['in']),
            _srt_timecode(s['out']),
            s['value'].replace('\n', '\r\n').strip()
        )

    return codecs.BOM_UTF8 + srt.encode('utf-8')



#!/usr/bin/python
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import print_function

import re

from .js import minify
from .utils import json


def load(f):
    return loads(f.read())

def loads(source):
    try:
        minified = minify(source)
        return json.loads(minified)
    except ValueError as e:
        msg = e.message if hasattr(e, 'message') else str(e)
        lineno = None
        colno = None
        try:
            m = re.search(r'line (\d+) column (\d+)', msg)
            if m:
                (lineno, colno) = [int(n) for n in m.groups()]
        except:
            pass
        if lineno and colno:
            s = minified.split('\n')
            context = s[lineno-1][max(0, colno-30):colno+30]
            msg += ' at:\n\n %s\n %s\033[1m^\033[0m' % (context, ' ' * (colno - max(0, colno-30) - 2))
        raise ValueError(msg)



##
#
# bencode.py python3 compatable bencode / bdecode
#
##

class Decoder(object):

    def _decode_int(self):
        """
        decode integer from bytearray
        return int
        """
        self.idx += 1
        start = self.idx
        end = self.data.index(b'e', self.idx)
        self.idx = end + 1
        return int(self.data[start:end])

    def _decode_str(self):
        """
        decode string from bytearray
        return string
        """
        start = self.data.index(b':', self.idx)
        l = int(self.data[self.idx:start].decode(), 10)
        if l < 0:
            raise Exception('invalid string size: %d' % l)
        start += 1
        ret = self.data[start:start+l]
        try:
            ret = ret.decode('utf-8')
        except:
            pass
        self.idx = start + l
        return ret

    def _decode_list(self):
        """
        decode list from bytearray
        return list
        """
        ls = []
        self.idx += 1
        while self.data[self.idx] != ord(b'e'):
            ls.append(self._decode())
        self.idx += 1
        return ls

    def _decode_dict(self):
        """
        decode dict from bytearray
        return dict
        """
        d = {}
        self.idx += 1
        while self.data[self.idx] != ord(b'e'):
            k = self._decode_str()
            v = self._decode()
            d[k] = v
        self.idx += 1
        return d

    def _decode(self):
        ch = chr(self.data[self.idx])
        if ch == 'l':
            return self._decode_list()
        elif ch == 'i':
            return self._decode_int()
        elif ch == 'd':
            return self._decode_dict()
        elif ch.isdigit():
            return self._decode_str()
        else:
            raise Exception('could not decode data: %s' % data)

    def decode(self, data):
        self.idx = 0
        self.data = data
        obj = self._decode()
        if len(data) != self.idx:
            raise Exception('failed to decode, extra data: %s' % data)
        return obj

def bdecode(data):
    """
    decode a bytearray
    return decoded object
    """
    return Decoder().decode(data)

def _encode_str(s, buff):
    """
    encode string to a buffer
    """
    s = bytearray(s)
    l = len(s)
    buff.append(bytearray(str(l)+':', 'utf-8'))
    buff.append(s)

def _encode_int(i, buff):
    """
    encode integer to a buffer
    """
    buff.append(b'i')
    buff.append(bytearray(str(i), 'ascii'))
    buff.append(b'e')

def _encode_list(l, buff):
    """
    encode list of elements to a buffer
    """
    buff.append(b'l')
    for i in l:
        _encode(i, buff)
    buff.append(b'e')

def _encode_dict(d, buff):
    """
    encode dict
    """
    buff.append(b'd')
    for k in sorted(d):
        if not isinstance(k, (bytes, str)):
            k = str(k)
        _encode(k, buff)
        _encode(d[k], buff)
    buff.append(b'e')

def _encode(obj, buff):
    """
    encode element obj to a buffer buff
    """
    if isinstance(obj, str):
        _encode_str(bytearray(obj, 'utf-8'), buff)
    elif isinstance(obj, bytes):
        _encode_str(bytearray(obj), buff)
    elif isinstance(obj, bytearray):
        _encode_str(obj, buff)
    elif str(obj).isdigit():
        _encode_int(obj, buff)
    elif isinstance(obj, int):
        _encode_int(obj, buff)
    elif isinstance(obj, list):
        _encode_list(obj, buff)
    elif hasattr(obj, 'keys') and hasattr(obj, 'values'):
        _encode_dict(obj, buff)
    elif str(obj) in ['True', 'False']:
        _encode_int(int(obj and '1' or '0'), buff)
    else:
        raise Exception('non serializable object: %s [%s]' % (obj, type(obj)))


def bencode(obj):
    """
    bencode element, return bytearray
    """
    buff = []
    _encode(obj, buff)
    ret = bytearray()
    for ba in buff:
        ret += ba
    return bytes(ret)





def tree(obj, indent=""):
    if not obj:
        return
    if isinstance(obj, dict):
        for key in obj:
            print(indent, key)
            tree(obj[key], indent + " ")



def find(obj, value, path=""):
    if not obj:
        return
    if isinstance(obj, list):
        for i, v in enumerate(obj):
            find(v, value, path + '[%s]' )
    elif isinstance(obj, dict):
        for key in obj:
            if isinstance(obj[key], str) and obj[key] == value:
                print(path, key, value)
            else:
                find(obj[key], value, "%s['%s']" % (path, key))



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import print_function

import re
from ox.net import read_url

def get_poster_url(id):
    url = 'http://piratecinema.org/posters/'
    html = read_url(url).decode('utf-8')
    results = re.compile('src="(.+)" title=".+\((\d{6}\d+)\)"').findall(html)
    for result in results:
        if result[1] == id:
            return url + result[0]
    return ''

if __name__ == '__main__':
    print(get_poster_url('0749451'))




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
# GPL 2008
from __future__ import print_function

import math

__all__ = ['get_country', 'get_country_name', 'normalize_country_name']

def update_countries():
    '''
        to update list of countries run the following command in python-ox
        echo "import ox.geo;ox.geo.update_countries()" | python
    '''
    import re
    import json
    from .net import read_url

    COUNTRIES = json.loads(read_url('http://oxjs.org/source/Ox.Geo/json/Ox.Geo.json'))
    countries = {}
    for country in COUNTRIES:
        #only existing countres have 2 codes
        if True or len(country['code']) == 2:
            countries[country['code']] = {
                "name": country['name'],
                "region": country['region'],
                "continent": country['continent'],
            }
            for key in ('googleName', 'imdbName'):
                if key in country:
                    if not 'aliases' in countries[country['code']]:
                        countries[country['code']]['aliases'] = []
                    if country[key] not in countries[country['code']]['aliases']:
                        countries[country['code']]['aliases'].append(country[key])

    data = json.dumps(countries, indent=4, ensure_ascii=False).encode('utf-8')
    with open('ox/geo.py') as f:
        pydata = f.read()
    pydata = re.sub(
        re.compile('\nCOUNTRIES = {.*?}\n\n', re.DOTALL),
        '\nCOUNTRIES = %s\n\n' % data, pydata)

    with open('ox/geo.py', 'w') as f:
        f.write(pydata)
    print('ox/geo.py updated')

COUNTRIES = {
    u"GE-AB": {
        u"region": u"Western Asia", 
        u"name": u"Abkhazia", 
        u"continent": u"Asia"
    }, 
    u"CW": {
        u"region": u"Caribbean", 
        u"name": u"Curaçao", 
        u"continent": u"South America"
    }, 
    u"GW": {
        u"region": u"Western Africa", 
        u"name": u"Guinea-Bissau", 
        u"continent": u"Africa"
    }, 
    u"GU": {
        u"region": u"Micronesia", 
        u"name": u"Guam", 
        u"continent": u"Oceania"
    }, 
    u"GT": {
        u"region": u"Central America", 
        u"name": u"Guatemala", 
        u"continent": u"South America"
    }, 
    u"GS": {
        u"region": u"Antarctica", 
        u"name": u"South Georgia and the South Sandwich Islands", 
        u"continent": u"Antarctica"
    }, 
    u"GR": {
        u"region": u"Southern Europe", 
        u"name": u"Greece", 
        u"continent": u"Europe"
    }, 
    u"GQ": {
        u"region": u"Middle Africa", 
        u"name": u"Equatorial Guinea", 
        u"continent": u"Africa"
    }, 
    u"GP": {
        u"region": u"Caribbean", 
        u"name": u"Guadeloupe", 
        u"continent": u"South America"
    }, 
    u"KAKH": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Kampuchea", 
        u"continent": u"Asia"
    }, 
    u"GY": {
        u"region": u"Southern America", 
        u"name": u"Guyana", 
        u"continent": u"South America"
    }, 
    u"GG": {
        u"region": u"Northern Europe", 
        u"name": u"Guernsey", 
        u"continent": u"Europe"
    }, 
    u"BYAA": {
        u"region": u"Eastern Europe", 
        u"name": u"Byelorussian Soviet Socialist Republic", 
        u"continent": u"Europe"
    }, 
    u"GE": {
        u"region": u"Western Asia", 
        u"name": u"Georgia", 
        u"continent": u"Asia"
    }, 
    u"GD": {
        u"region": u"Caribbean", 
        u"name": u"Grenada", 
        u"continent": u"South America"
    }, 
    u"GB": {
        u"region": u"Northern Europe", 
        u"aliases": [
            u"UK"
        ], 
        u"name": u"United Kingdom", 
        u"continent": u"Europe"
    }, 
    u"GA": {
        u"region": u"Middle Africa", 
        u"name": u"Gabon", 
        u"continent": u"Africa"
    }, 
    u"YEYE": {
        u"region": u"Western Asia", 
        u"name": u"North Yemen", 
        u"continent": u"Asia"
    }, 
    u"GN": {
        u"region": u"Western Africa", 
        u"name": u"Guinea", 
        u"continent": u"Africa"
    }, 
    u"GM": {
        u"region": u"Western Africa", 
        u"aliases": [
            u"The Gambia"
        ], 
        u"name": u"Gambia", 
        u"continent": u"Africa"
    }, 
    u"GL": {
        u"region": u"Northern America", 
        u"name": u"Greenland", 
        u"continent": u"North America"
    }, 
    u"GI": {
        u"region": u"Southern Europe", 
        u"name": u"Gibraltar", 
        u"continent": u"Europe"
    }, 
    u"GH": {
        u"region": u"Western Africa", 
        u"name": u"Ghana", 
        u"continent": u"Africa"
    }, 
    u"SUHH": {
        u"region": u"Eastern Europe", 
        u"name": u"Soviet Union", 
        u"continent": u"Europe"
    }, 
    u"JTUM": {
        u"region": u"Polynesia", 
        u"name": u"Johnston Island", 
        u"continent": u"Oceania"
    }, 
    u"EH": {
        u"region": u"Northern Africa", 
        u"aliases": [
            u"Western Sahara"
        ], 
        u"name": u"Sahrawi", 
        u"continent": u"Africa"
    }, 
    u"ANHH": {
        u"region": u"Caribbean", 
        u"name": u"Netherlands Antilles", 
        u"continent": u"South America"
    }, 
    u"AE-RK": {
        u"region": u"Western Asia", 
        u"name": u"Ras al-Khaimah", 
        u"continent": u"Asia"
    }, 
    u"ZA": {
        u"region": u"Southern Africa", 
        u"name": u"South Africa", 
        u"continent": u"Africa"
    }, 
    u"GB-WLS": {
        u"region": u"Northern Europe", 
        u"name": u"Wales", 
        u"continent": u"Europe"
    }, 
    u"ZW": {
        u"region": u"Eastern Africa", 
        u"name": u"Zimbabwe", 
        u"continent": u"Africa"
    }, 
    u"YUCS": {
        u"region": u"Southern Europe", 
        u"aliases": [
            u"Federal Republic of Yugoslavia"
        ], 
        u"name": u"Yugoslavia", 
        u"continent": u"Europe"
    }, 
    u"ME": {
        u"region": u"Southern Europe", 
        u"name": u"Montenegro", 
        u"continent": u"Europe"
    }, 
    u"MD": {
        u"region": u"Eastern Europe", 
        u"name": u"Moldova", 
        u"continent": u"Europe"
    }, 
    u"MG": {
        u"region": u"Eastern Africa", 
        u"name": u"Madagascar", 
        u"continent": u"Africa"
    }, 
    u"MF": {
        u"region": u"Caribbean", 
        u"aliases": [
            u"Saint Martin (French part)"
        ], 
        u"name": u"Saint Martin", 
        u"continent": u"South America"
    }, 
    u"MA": {
        u"region": u"Northern Africa", 
        u"name": u"Morocco", 
        u"continent": u"Africa"
    }, 
    u"MC": {
        u"region": u"Western Europe", 
        u"name": u"Monaco", 
        u"continent": u"Europe"
    }, 
    u"MM": {
        u"region": u"South-Eastern Asia", 
        u"aliases": [
            u"Burma"
        ], 
        u"name": u"Myanmar", 
        u"continent": u"Asia"
    }, 
    u"ML": {
        u"region": u"Western Africa", 
        u"name": u"Mali", 
        u"continent": u"Africa"
    }, 
    u"MO": {
        u"region": u"Eastern Asia", 
        u"aliases": [
            u"Macao"
        ], 
        u"name": u"Macau", 
        u"continent": u"Asia"
    }, 
    u"MN": {
        u"region": u"Eastern Asia", 
        u"name": u"Mongolia", 
        u"continent": u"Asia"
    }, 
    u"AE-UQ": {
        u"region": u"Western Asia", 
        u"name": u"Umm al-Quwain", 
        u"continent": u"Asia"
    }, 
    u"MH": {
        u"region": u"Micronesia", 
        u"name": u"Marshall Islands", 
        u"continent": u"Oceania"
    }, 
    u"MK": {
        u"region": u"Southern Europe", 
        u"aliases": [
            u"Former Yugoslav Republic of Macedonia", 
            u"Republic of Macedonia"
        ], 
        u"name": u"Macedonia", 
        u"continent": u"Europe"
    }, 
    u"MU": {
        u"region": u"Eastern Africa", 
        u"name": u"Mauritius", 
        u"continent": u"Africa"
    }, 
    u"MT": {
        u"region": u"Southern Europe", 
        u"name": u"Malta", 
        u"continent": u"Europe"
    }, 
    u"MW": {
        u"region": u"Eastern Africa", 
        u"name": u"Malawi", 
        u"continent": u"Africa"
    }, 
    u"MV": {
        u"region": u"Southern Asia", 
        u"name": u"Maldives", 
        u"continent": u"Asia"
    }, 
    u"MQ": {
        u"region": u"Caribbean", 
        u"name": u"Martinique", 
        u"continent": u"South America"
    }, 
    u"MP": {
        u"region": u"Micronesia", 
        u"name": u"Northern Mariana Islands", 
        u"continent": u"Oceania"
    }, 
    u"MS": {
        u"region": u"Caribbean", 
        u"name": u"Montserrat", 
        u"continent": u"South America"
    }, 
    u"MR": {
        u"region": u"Western Africa", 
        u"name": u"Mauritania", 
        u"continent": u"Africa"
    }, 
    u"MY": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Malaysia", 
        u"continent": u"Asia"
    }, 
    u"MX": {
        u"region": u"Central America", 
        u"name": u"Mexico", 
        u"continent": u"South America"
    }, 
    u"MZ": {
        u"region": u"Eastern Africa", 
        u"name": u"Mozambique", 
        u"continent": u"Africa"
    }, 
    u"FR": {
        u"region": u"Western Europe", 
        u"name": u"France", 
        u"continent": u"Europe"
    }, 
    u"ZRCD": {
        u"region": u"Middle Africa", 
        u"name": u"Zaire", 
        u"continent": u"Africa"
    }, 
    u"ZA-BO": {
        u"region": u"Southern Africa", 
        u"name": u"Bophuthatswana", 
        u"continent": u"Africa"
    }, 
    u"FI": {
        u"region": u"Northern Europe", 
        u"name": u"Finland", 
        u"continent": u"Europe"
    }, 
    u"FJ": {
        u"region": u"Melanesia", 
        u"name": u"Fiji", 
        u"continent": u"Oceania"
    }, 
    u"FK": {
        u"region": u"Southern America", 
        u"name": u"Falkland Islands", 
        u"continent": u"South America"
    }, 
    u"FM": {
        u"region": u"Micronesia", 
        u"aliases": [
            u"Federated States of Micronesia"
        ], 
        u"name": u"Micronesia", 
        u"continent": u"Oceania"
    }, 
    u"FO": {
        u"region": u"Northern Europe", 
        u"name": u"Faroe Islands", 
        u"continent": u"Europe"
    }, 
    u"SITH": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Siam", 
        u"continent": u"Asia"
    }, 
    u"NHVU": {
        u"region": u"Melanesia", 
        u"name": u"New Hebrides", 
        u"continent": u"Oceania"
    }, 
    u"AR-AQ": {
        u"region": u"Antarctica", 
        u"name": u"Argentine Antarctica", 
        u"continent": u"Antarctica"
    }, 
    u"FR-AQ": {
        u"region": u"Antarctica", 
        u"name": u"Adélie Land", 
        u"continent": u"Antarctica"
    }, 
    u"NHVU-VE": {
        u"region": u"Melanesia", 
        u"name": u"Vemerana", 
        u"continent": u"Oceania"
    }, 
    u"SZ": {
        u"region": u"Southern Africa", 
        u"name": u"Swaziland", 
        u"continent": u"Africa"
    }, 
    u"SY": {
        u"region": u"Western Asia", 
        u"name": u"Syria", 
        u"continent": u"Asia"
    }, 
    u"SX": {
        u"region": u"Caribbean", 
        u"name": u"Sint Maarten", 
        u"continent": u"South America"
    }, 
    u"SS": {
        u"region": u"Northern Africa", 
        u"name": u"South Sudan", 
        u"continent": u"Africa"
    }, 
    u"SR": {
        u"region": u"Southern America", 
        u"name": u"Suriname", 
        u"continent": u"South America"
    }, 
    u"SV": {
        u"region": u"Central America", 
        u"name": u"El Salvador", 
        u"continent": u"South America"
    }, 
    u"ST": {
        u"region": u"Middle Africa", 
        u"aliases": [
            u"Sao Tome and Principe"
        ], 
        u"name": u"São Tomé and Príncipe", 
        u"continent": u"Africa"
    }, 
    u"SK": {
        u"region": u"Eastern Europe", 
        u"name": u"Slovakia", 
        u"continent": u"Europe"
    }, 
    u"SJ": {
        u"region": u"Northern Europe", 
        u"name": u"Svalbard and Jan Mayen", 
        u"continent": u"Europe"
    }, 
    u"SI": {
        u"region": u"Southern Europe", 
        u"name": u"Slovenia", 
        u"continent": u"Europe"
    }, 
    u"SH": {
        u"region": u"Western Africa", 
        u"aliases": [
            u"Saint Helena"
        ], 
        u"name": u"Saint Helena, Ascension and Tristan da Cunha", 
        u"continent": u"Africa"
    }, 
    u"SO": {
        u"region": u"Eastern Africa", 
        u"name": u"Somalia", 
        u"continent": u"Africa"
    }, 
    u"SN": {
        u"region": u"Western Africa", 
        u"name": u"Senegal", 
        u"continent": u"Africa"
    }, 
    u"SM": {
        u"region": u"Southern Europe", 
        u"name": u"San Marino", 
        u"continent": u"Europe"
    }, 
    u"SL": {
        u"region": u"Western Africa", 
        u"name": u"Sierra Leone", 
        u"continent": u"Africa"
    }, 
    u"SC": {
        u"region": u"Eastern Africa", 
        u"name": u"Seychelles", 
        u"continent": u"Africa"
    }, 
    u"SB": {
        u"region": u"Melanesia", 
        u"name": u"Solomon Islands", 
        u"continent": u"Oceania"
    }, 
    u"SA": {
        u"region": u"Western Asia", 
        u"name": u"Saudi Arabia", 
        u"continent": u"Asia"
    }, 
    u"SG": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Singapore", 
        u"continent": u"Asia"
    }, 
    u"SE": {
        u"region": u"Northern Europe", 
        u"name": u"Sweden", 
        u"continent": u"Europe"
    }, 
    u"SD": {
        u"region": u"Northern Africa", 
        u"name": u"Sudan", 
        u"continent": u"Africa"
    }, 
    u"YE": {
        u"region": u"Western Asia", 
        u"name": u"Yemen", 
        u"continent": u"Asia"
    }, 
    u"YT": {
        u"region": u"Eastern Africa", 
        u"name": u"Mayotte", 
        u"continent": u"Africa"
    }, 
    u"LB": {
        u"region": u"Western Asia", 
        u"name": u"Lebanon", 
        u"continent": u"Asia"
    }, 
    u"LC": {
        u"region": u"Caribbean", 
        u"name": u"Saint Lucia", 
        u"continent": u"South America"
    }, 
    u"LA": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Laos", 
        u"continent": u"Asia"
    }, 
    u"ZA-TR": {
        u"region": u"Southern Africa", 
        u"name": u"Transkei", 
        u"continent": u"Africa"
    }, 
    u"LK": {
        u"region": u"Southern Asia", 
        u"name": u"Sri Lanka", 
        u"continent": u"Asia"
    }, 
    u"LI": {
        u"region": u"Western Europe", 
        u"name": u"Liechtenstein", 
        u"continent": u"Europe"
    }, 
    u"LV": {
        u"region": u"Northern Europe", 
        u"name": u"Latvia", 
        u"continent": u"Europe"
    }, 
    u"LT": {
        u"region": u"Northern Europe", 
        u"name": u"Lithuania", 
        u"continent": u"Europe"
    }, 
    u"LU": {
        u"region": u"Western Europe", 
        u"name": u"Luxembourg", 
        u"continent": u"Europe"
    }, 
    u"PG-NSA": {
        u"region": u"Melanesia", 
        u"name": u"Bougainville", 
        u"continent": u"Oceania"
    }, 
    u"LS": {
        u"region": u"Southern Africa", 
        u"name": u"Lesotho", 
        u"continent": u"Africa"
    }, 
    u"LY": {
        u"region": u"Northern Africa", 
        u"name": u"Libya", 
        u"continent": u"Africa"
    }, 
    u"DEDE": {
        u"region": u"Western Europe", 
        u"name": u"West Germany", 
        u"continent": u"Europe"
    }, 
    u"GF": {
        u"region": u"Southern America", 
        u"name": u"French Guiana", 
        u"continent": u"South America"
    }, 
    u"AU-CS": {
        u"region": u"Australia and New Zealand", 
        u"name": u"Coral Sea Islands", 
        u"continent": u"Oceania"
    }, 
    u"WKUM": {
        u"region": u"Micronesia", 
        u"name": u"Wake Island", 
        u"continent": u"Oceania"
    }, 
    u"UAUA": {
        u"region": u"Eastern Europe", 
        u"name": u"Ukrainian Soviet Socialist Republic", 
        u"continent": u"Europe"
    }, 
    u"CTKI": {
        u"region": u"Micronesia", 
        u"name": u"Canton and Enderbury Islands", 
        u"continent": u"Oceania"
    }, 
    u"RU": {
        u"region": u"Eastern Europe", 
        u"name": u"Russia", 
        u"continent": u"Europe"
    }, 
    u"RW": {
        u"region": u"Eastern Africa", 
        u"name": u"Rwanda", 
        u"continent": u"Africa"
    }, 
    u"RS": {
        u"region": u"Southern Europe", 
        u"name": u"Serbia", 
        u"continent": u"Europe"
    }, 
    u"RE": {
        u"region": u"Eastern Africa", 
        u"name": u"Réunion", 
        u"continent": u"Africa"
    }, 
    u"LR": {
        u"region": u"Western Africa", 
        u"name": u"Liberia", 
        u"continent": u"Africa"
    }, 
    u"RO": {
        u"region": u"Eastern Europe", 
        u"name": u"Romania", 
        u"continent": u"Europe"
    }, 
    u"PK-NA": {
        u"region": u"Southern Asia", 
        u"name": u"Gilgit-Baltistan", 
        u"continent": u"Asia"
    }, 
    u"GG-HE": {
        u"region": u"Northern Europe", 
        u"name": u"Herm", 
        u"continent": u"Europe"
    }, 
    u"CSXX": {
        u"region": u"Southern Europe", 
        u"name": u"Serbia and Montenegro", 
        u"continent": u"Europe"
    }, 
    u"AU-AC": {
        u"region": u"Australia and New Zealand", 
        u"name": u"Ashmore and Cartier Islands", 
        u"continent": u"Oceania"
    }, 
    u"AU-AQ": {
        u"region": u"Antarctica", 
        u"name": u"Australian Antarctic Territory", 
        u"continent": u"Antarctica"
    }, 
    u"TPTL": {
        u"region": u"South-Eastern Asia", 
        u"name": u"East Timor", 
        u"continent": u"Asia"
    }, 
    u"GEKI": {
        u"region": u"Micronesia", 
        u"name": u"Gilbert Islands", 
        u"continent": u"Oceania"
    }, 
    u"NQAQ": {
        u"region": u"Antarctica", 
        u"name": u"Queen Maud Land", 
        u"continent": u"Antarctica"
    }, 
    u"EE": {
        u"region": u"Northern Europe", 
        u"name": u"Estonia", 
        u"continent": u"Europe"
    }, 
    u"EG": {
        u"region": u"Northern Africa", 
        u"name": u"Egypt", 
        u"continent": u"Africa"
    }, 
    u"EA": {
        u"region": u"Northern Africa", 
        u"name": u"Ceuta and Melilla", 
        u"continent": u"Africa"
    }, 
    u"EC": {
        u"region": u"Southern America", 
        u"name": u"Ecuador", 
        u"continent": u"South America"
    }, 
    u"EU": {
        u"region": u"Western Europe", 
        u"name": u"European Union", 
        u"continent": u"Europe"
    }, 
    u"ET": {
        u"region": u"Eastern Africa", 
        u"name": u"Ethiopia", 
        u"continent": u"Africa"
    }, 
    u"ES": {
        u"region": u"Southern Europe", 
        u"name": u"Spain", 
        u"continent": u"Europe"
    }, 
    u"ER": {
        u"region": u"Eastern Africa", 
        u"name": u"Eritrea", 
        u"continent": u"Africa"
    }, 
    u"RU-CE": {
        u"region": u"Eastern Europe", 
        u"name": u"Chechnia", 
        u"continent": u"Europe"
    }, 
    u"VU": {
        u"region": u"Melanesia", 
        u"name": u"Vanuatu", 
        u"continent": u"Oceania"
    }, 
    u"AIDJ": {
        u"region": u"Eastern Africa", 
        u"name": u"French Afar and Issas", 
        u"continent": u"Africa"
    }, 
    u"IN": {
        u"region": u"Southern Asia", 
        u"name": u"India", 
        u"continent": u"Asia"
    }, 
    u"XK": {
        u"region": u"Southern Europe", 
        u"aliases": [
            u"Kosova (Kosovo)"
        ], 
        u"name": u"Kosovo", 
        u"continent": u"Europe"
    }, 
    u"PK-JK": {
        u"region": u"Southern Asia", 
        u"name": u"Azad Kashmir", 
        u"continent": u"Asia"
    }, 
    u"BUMM": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Burma", 
        u"continent": u"Asia"
    }, 
    u"NR": {
        u"region": u"Micronesia", 
        u"name": u"Nauru", 
        u"continent": u"Oceania"
    }, 
    u"KG": {
        u"region": u"Central Asia", 
        u"name": u"Kyrgyzstan", 
        u"continent": u"Asia"
    }, 
    u"KE": {
        u"region": u"Eastern Africa", 
        u"name": u"Kenya", 
        u"continent": u"Africa"
    }, 
    u"KI": {
        u"region": u"Micronesia", 
        u"name": u"Kiribati", 
        u"continent": u"Oceania"
    }, 
    u"KH": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Cambodia", 
        u"continent": u"Asia"
    }, 
    u"KN": {
        u"region": u"Caribbean", 
        u"name": u"Saint Kitts and Nevis", 
        u"continent": u"South America"
    }, 
    u"KM": {
        u"region": u"Eastern Africa", 
        u"name": u"Comoros", 
        u"continent": u"Africa"
    }, 
    u"KR": {
        u"region": u"Eastern Asia", 
        u"name": u"South Korea", 
        u"continent": u"Asia"
    }, 
    u"KP": {
        u"region": u"Eastern Asia", 
        u"name": u"North Korea", 
        u"continent": u"Asia"
    }, 
    u"KW": {
        u"region": u"Western Asia", 
        u"name": u"Kuwait", 
        u"continent": u"Asia"
    }, 
    u"KZ": {
        u"region": u"Central Asia", 
        u"name": u"Kazakhstan", 
        u"continent": u"Asia"
    }, 
    u"KY": {
        u"region": u"Caribbean", 
        u"name": u"Cayman Islands", 
        u"continent": u"South America"
    }, 
    u"DO": {
        u"region": u"Caribbean", 
        u"name": u"Dominican Republic", 
        u"continent": u"South America"
    }, 
    u"DM": {
        u"region": u"Caribbean", 
        u"name": u"Dominica", 
        u"continent": u"South America"
    }, 
    u"DJ": {
        u"region": u"Eastern Africa", 
        u"name": u"Djibouti", 
        u"continent": u"Africa"
    }, 
    u"DK": {
        u"region": u"Northern Europe", 
        u"name": u"Denmark", 
        u"continent": u"Europe"
    }, 
    u"DG": {
        u"region": u"Southern Asia", 
        u"name": u"Diego Garcia", 
        u"continent": u"Asia"
    }, 
    u"DE": {
        u"region": u"Western Europe", 
        u"name": u"Germany", 
        u"continent": u"Europe"
    }, 
    u"DZ": {
        u"region": u"Northern Africa", 
        u"name": u"Algeria", 
        u"continent": u"Africa"
    }, 
    u"BQAQ": {
        u"region": u"Antarctica", 
        u"name": u"British Antarctic Territory", 
        u"continent": u"Antarctica"
    }, 
    u"ZA-CI": {
        u"region": u"Southern Africa", 
        u"name": u"Ciskei", 
        u"continent": u"Africa"
    }, 
    u"GB-SL": {
        u"region": u"Northern Europe", 
        u"name": u"Sealand", 
        u"continent": u"Europe"
    }, 
    u"MD-SN": {
        u"region": u"Eastern Europe", 
        u"name": u"Transnistria", 
        u"continent": u"Europe"
    }, 
    u"SKIN": {
        u"region": u"Southern Asia", 
        u"name": u"Sikkim", 
        u"continent": u"Asia"
    }, 
    u"FXFR": {
        u"region": u"Western Europe", 
        u"name": u"Metropolitan France", 
        u"continent": u"Europe"
    }, 
    u"AE-FU": {
        u"region": u"Western Asia", 
        u"name": u"Fujairah", 
        u"continent": u"Asia"
    }, 
    u"QA": {
        u"region": u"Western Asia", 
        u"name": u"Qatar", 
        u"continent": u"Asia"
    }, 
    u"WF": {
        u"region": u"Polynesia", 
        u"name": u"Wallis and Futuna", 
        u"continent": u"Oceania"
    }, 
    u"JP": {
        u"region": u"Eastern Asia", 
        u"name": u"Japan", 
        u"continent": u"Asia"
    }, 
    u"JM": {
        u"region": u"Caribbean", 
        u"name": u"Jamaica", 
        u"continent": u"South America"
    }, 
    u"JO": {
        u"region": u"Western Asia", 
        u"name": u"Jordan", 
        u"continent": u"Asia"
    }, 
    u"WS": {
        u"region": u"Polynesia", 
        u"name": u"Samoa", 
        u"continent": u"Oceania"
    }, 
    u"JE": {
        u"region": u"Northern Europe", 
        u"name": u"Jersey", 
        u"continent": u"Europe"
    }, 
    u"KM-M": {
        u"region": u"Eastern Africa", 
        u"name": u"Mohéli", 
        u"continent": u"Africa"
    }, 
    u"KM-A": {
        u"region": u"Eastern Africa", 
        u"name": u"Anjouan", 
        u"continent": u"Africa"
    }, 
    u"PZPA": {
        u"region": u"Central America", 
        u"name": u"Panama Canal Zone", 
        u"continent": u"South America"
    }, 
    u"MIUM": {
        u"region": u"Polynesia", 
        u"name": u"Midway Islands", 
        u"continent": u"Oceania"
    }, 
    u"GEHH": {
        u"region": u"Micronesia", 
        u"name": u"Gilbert and Ellice Islands", 
        u"continent": u"Oceania"
    }, 
    u"NZ-AQ": {
        u"region": u"Antarctica", 
        u"name": u"Ross Dependency", 
        u"continent": u"Antarctica"
    }, 
    u"HVBF": {
        u"region": u"Western Africa", 
        u"name": u"Upper Volta", 
        u"continent": u"Africa"
    }, 
    u"GB-AD": {
        u"region": u"Western Asia", 
        u"name": u"Akrotiri and Dhekelia", 
        u"continent": u"Asia"
    }, 
    u"UG-RW": {
        u"region": u"Eastern Africa", 
        u"name": u"Rwenzururu", 
        u"continent": u"Africa"
    }, 
    u"ZM": {
        u"region": u"Eastern Africa", 
        u"name": u"Zambia", 
        u"continent": u"Africa"
    }, 
    u"NTHH": {
        u"region": u"Western Asia", 
        u"name": u"Neutral Zone", 
        u"continent": u"Asia"
    }, 
    u"PR": {
        u"region": u"Caribbean", 
        u"name": u"Puerto Rico", 
        u"continent": u"South America"
    }, 
    u"PS": {
        u"region": u"Western Asia", 
        u"aliases": [
            u"Palestinian Territories", 
            u"Occupied Palestinian Territory"
        ], 
        u"name": u"Palestine", 
        u"continent": u"Asia"
    }, 
    u"PW": {
        u"region": u"Micronesia", 
        u"name": u"Palau", 
        u"continent": u"Oceania"
    }, 
    u"PT": {
        u"region": u"Southern Europe", 
        u"name": u"Portugal", 
        u"continent": u"Europe"
    }, 
    u"PY": {
        u"region": u"Southern America", 
        u"name": u"Paraguay", 
        u"continent": u"South America"
    }, 
    u"PA": {
        u"region": u"Central America", 
        u"name": u"Panama", 
        u"continent": u"South America"
    }, 
    u"PF": {
        u"region": u"Polynesia", 
        u"name": u"French Polynesia", 
        u"continent": u"Oceania"
    }, 
    u"PG": {
        u"region": u"Melanesia", 
        u"name": u"Papua New Guinea", 
        u"continent": u"Oceania"
    }, 
    u"PE": {
        u"region": u"Southern America", 
        u"name": u"Peru", 
        u"continent": u"South America"
    }, 
    u"PK": {
        u"region": u"Southern Asia", 
        u"name": u"Pakistan", 
        u"continent": u"Asia"
    }, 
    u"PH": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Philippines", 
        u"continent": u"Asia"
    }, 
    u"PN": {
        u"region": u"Polynesia", 
        u"aliases": [
            u"Pitcairn"
        ], 
        u"name": u"Pitcairn Islands", 
        u"continent": u"Oceania"
    }, 
    u"PL": {
        u"region": u"Eastern Europe", 
        u"name": u"Poland", 
        u"continent": u"Europe"
    }, 
    u"PM": {
        u"region": u"Northern America", 
        u"name": u"Saint Pierre and Miquelon", 
        u"continent": u"North America"
    }, 
    u"VDVN": {
        u"region": u"South-Eastern Asia", 
        u"name": u"North Vietnam", 
        u"continent": u"Asia"
    }, 
    u"NO-PI": {
        u"region": u"Antarctica", 
        u"name": u"Peter I Island", 
        u"continent": u"Antarctica"
    }, 
    u"KOJP": {
        u"region": u"Eastern Asia", 
        u"name": u"Korea", 
        u"continent": u"Asia"
    }, 
    u"GBBZ": {
        u"region": u"Central America", 
        u"name": u"British Honduras", 
        u"continent": u"South America"
    }, 
    u"RHZW-ZR": {
        u"region": u"Eastern Africa", 
        u"name": u"Zimbabwe Rhodesia", 
        u"continent": u"Africa"
    }, 
    u"GB-NIR": {
        u"region": u"Northern Europe", 
        u"name": u"Northern Ireland", 
        u"continent": u"Europe"
    }, 
    u"NG-BI": {
        u"region": u"Western Africa", 
        u"name": u"Biafra", 
        u"continent": u"Africa"
    }, 
    u"CK": {
        u"region": u"Polynesia", 
        u"name": u"Cook Islands", 
        u"continent": u"Oceania"
    }, 
    u"CI": {
        u"region": u"Western Africa", 
        u"aliases": [
            u"Ivory Coast"
        ], 
        u"name": u"Côte d'Ivoire", 
        u"continent": u"Africa"
    }, 
    u"CH": {
        u"region": u"Western Europe", 
        u"name": u"Switzerland", 
        u"continent": u"Europe"
    }, 
    u"CO": {
        u"region": u"Southern America", 
        u"name": u"Colombia", 
        u"continent": u"South America"
    }, 
    u"CN": {
        u"region": u"Eastern Asia", 
        u"name": u"China", 
        u"continent": u"Asia"
    }, 
    u"CM": {
        u"region": u"Middle Africa", 
        u"name": u"Cameroon", 
        u"continent": u"Africa"
    }, 
    u"CL-AQ": {
        u"region": u"Antarctica", 
        u"name": u"Chilean Antarctic Territory", 
        u"continent": u"Antarctica"
    }, 
    u"CC": {
        u"region": u"South-Eastern Asia", 
        u"aliases": [
            u"Cocos (Keeling) Islands"
        ], 
        u"name": u"Cocos Islands", 
        u"continent": u"Asia"
    }, 
    u"CA": {
        u"region": u"Northern America", 
        u"name": u"Canada", 
        u"continent": u"North America"
    }, 
    u"CG": {
        u"region": u"Middle Africa", 
        u"aliases": [
            u"Congo"
        ], 
        u"name": u"Republic of the Congo", 
        u"continent": u"Africa"
    }, 
    u"CF": {
        u"region": u"Middle Africa", 
        u"name": u"Central African Republic", 
        u"continent": u"Africa"
    }, 
    u"CD": {
        u"region": u"Middle Africa", 
        u"aliases": [
            u"Democratic Republic of Congo"
        ], 
        u"name": u"Democratic Republic of the Congo", 
        u"continent": u"Africa"
    }, 
    u"CZ": {
        u"region": u"Eastern Europe", 
        u"name": u"Czech Republic", 
        u"continent": u"Europe"
    }, 
    u"CY": {
        u"region": u"Western Asia", 
        u"name": u"Cyprus", 
        u"continent": u"Asia"
    }, 
    u"CX": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Christmas Island", 
        u"continent": u"Asia"
    }, 
    u"CR": {
        u"region": u"Central America", 
        u"name": u"Costa Rica", 
        u"continent": u"South America"
    }, 
    u"CP": {
        u"region": u"Central America", 
        u"name": u"Clipperton Island", 
        u"continent": u"South America"
    }, 
    u"VNVN": {
        u"region": u"South-Eastern Asia", 
        u"name": u"South Vietnam", 
        u"continent": u"Asia"
    }, 
    u"CV": {
        u"region": u"Western Africa", 
        u"name": u"Cape Verde", 
        u"continent": u"Africa"
    }, 
    u"CU": {
        u"region": u"Caribbean", 
        u"name": u"Cuba", 
        u"continent": u"South America"
    }, 
    u"AO-CAB": {
        u"region": u"Middle Africa", 
        u"name": u"Cabinda", 
        u"continent": u"Africa"
    }, 
    u"GBKN": {
        u"region": u"Caribbean", 
        u"name": u"Saint Christopher-Nevis-Anguilla", 
        u"continent": u"South America"
    }, 
    u"LKLK": {
        u"region": u"Southern Asia", 
        u"name": u"Ceylon", 
        u"continent": u"Asia"
    }, 
    u"CSHH": {
        u"region": u"Eastern Europe", 
        u"name": u"Czechoslovakia", 
        u"continent": u"Europe"
    }, 
    u"AE-AZ": {
        u"region": u"Western Asia", 
        u"name": u"Abu Dhabi", 
        u"continent": u"Asia"
    }, 
    u"SO-SO": {
        u"region": u"Eastern Africa", 
        u"name": u"Somaliland", 
        u"continent": u"Africa"
    }, 
    u"AE-AJ": {
        u"region": u"Western Asia", 
        u"name": u"Ajman", 
        u"continent": u"Asia"
    }, 
    u"VA": {
        u"region": u"Southern Europe", 
        u"aliases": [
            u"Holy See (Vatican City State)"
        ], 
        u"name": u"Vatican City", 
        u"continent": u"Europe"
    }, 
    u"VC": {
        u"region": u"Caribbean", 
        u"name": u"Saint Vincent and the Grenadines", 
        u"continent": u"South America"
    }, 
    u"VE": {
        u"region": u"Southern America", 
        u"name": u"Venezuela", 
        u"continent": u"South America"
    }, 
    u"VG": {
        u"region": u"Caribbean", 
        u"name": u"British Virgin Islands", 
        u"continent": u"South America"
    }, 
    u"IQ": {
        u"region": u"Western Asia", 
        u"name": u"Iraq", 
        u"continent": u"Asia"
    }, 
    u"VI": {
        u"region": u"Caribbean", 
        u"aliases": [
            u"US Virgin Islands"
        ], 
        u"name": u"United States Virgin Islands", 
        u"continent": u"South America"
    }, 
    u"IS": {
        u"region": u"Northern Europe", 
        u"name": u"Iceland", 
        u"continent": u"Europe"
    }, 
    u"IR": {
        u"region": u"Southern Asia", 
        u"name": u"Iran", 
        u"continent": u"Asia"
    }, 
    u"IT": {
        u"region": u"Southern Europe", 
        u"name": u"Italy", 
        u"continent": u"Europe"
    }, 
    u"VN": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Vietnam", 
        u"continent": u"Asia"
    }, 
    u"IM": {
        u"region": u"Northern Europe", 
        u"name": u"Isle of Man", 
        u"continent": u"Europe"
    }, 
    u"IL": {
        u"region": u"Western Asia", 
        u"name": u"Israel", 
        u"continent": u"Asia"
    }, 
    u"IO": {
        u"region": u"Southern Asia", 
        u"name": u"British Indian Ocean Territory", 
        u"continent": u"Asia"
    }, 
    u"NHVU-TF": {
        u"region": u"Melanesia", 
        u"name": u"Tafea", 
        u"continent": u"Oceania"
    }, 
    u"IC": {
        u"region": u"Northern Africa", 
        u"name": u"Canary Islands", 
        u"continent": u"Africa"
    }, 
    u"IE": {
        u"region": u"Northern Europe", 
        u"name": u"Ireland", 
        u"continent": u"Europe"
    }, 
    u"ID": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Indonesia", 
        u"continent": u"Asia"
    }, 
    u"NHVU-TN": {
        u"region": u"Melanesia", 
        u"name": u"Tanna", 
        u"continent": u"Oceania"
    }, 
    u"GB-ENG": {
        u"region": u"Northern Europe", 
        u"name": u"England", 
        u"continent": u"Europe"
    }, 
    u"GG-AL": {
        u"region": u"Northern Europe", 
        u"name": u"Alderney", 
        u"continent": u"Europe"
    }, 
    u"BD": {
        u"region": u"Southern Asia", 
        u"name": u"Bangladesh", 
        u"continent": u"Asia"
    }, 
    u"BE": {
        u"region": u"Western Europe", 
        u"name": u"Belgium", 
        u"continent": u"Europe"
    }, 
    u"BF": {
        u"region": u"Western Africa", 
        u"name": u"Burkina Faso", 
        u"continent": u"Africa"
    }, 
    u"BG": {
        u"region": u"Eastern Europe", 
        u"name": u"Bulgaria", 
        u"continent": u"Europe"
    }, 
    u"BA": {
        u"region": u"Southern Europe", 
        u"name": u"Bosnia and Herzegovina", 
        u"continent": u"Europe"
    }, 
    u"BB": {
        u"region": u"Caribbean", 
        u"name": u"Barbados", 
        u"continent": u"South America"
    }, 
    u"AE-SH": {
        u"region": u"Western Asia", 
        u"name": u"Sharjah", 
        u"continent": u"Asia"
    }, 
    u"BL": {
        u"region": u"Caribbean", 
        u"name": u"Saint Barthélemy", 
        u"continent": u"South America"
    }, 
    u"BM": {
        u"region": u"Northern America", 
        u"name": u"Bermuda", 
        u"continent": u"North America"
    }, 
    u"BN": {
        u"region": u"South-Eastern Asia", 
        u"aliases": [
            u"Brunei Darussalam"
        ], 
        u"name": u"Brunei", 
        u"continent": u"Asia"
    }, 
    u"BO": {
        u"region": u"Southern America", 
        u"name": u"Bolivia", 
        u"continent": u"South America"
    }, 
    u"BH": {
        u"region": u"Western Asia", 
        u"name": u"Bahrain", 
        u"continent": u"Asia"
    }, 
    u"BI": {
        u"region": u"Eastern Africa", 
        u"name": u"Burundi", 
        u"continent": u"Africa"
    }, 
    u"BJ": {
        u"region": u"Western Africa", 
        u"name": u"Benin", 
        u"continent": u"Africa"
    }, 
    u"BT": {
        u"region": u"Southern Asia", 
        u"name": u"Bhutan", 
        u"continent": u"Asia"
    }, 
    u"BV": {
        u"region": u"Antarctica", 
        u"name": u"Bouvet Island", 
        u"continent": u"Antarctica"
    }, 
    u"BW": {
        u"region": u"Southern Africa", 
        u"name": u"Botswana", 
        u"continent": u"Africa"
    }, 
    u"BQ": {
        u"region": u"Caribbean", 
        u"name": u"Bonaire, Sint Eustatius and Saba", 
        u"continent": u"South America"
    }, 
    u"BR": {
        u"region": u"Southern America", 
        u"name": u"Brazil", 
        u"continent": u"South America"
    }, 
    u"BS": {
        u"region": u"Caribbean", 
        u"aliases": [
            u"The Bahamas"
        ], 
        u"name": u"Bahamas", 
        u"continent": u"South America"
    }, 
    u"BY": {
        u"region": u"Eastern Europe", 
        u"name": u"Belarus", 
        u"continent": u"Europe"
    }, 
    u"BZ": {
        u"region": u"Central America", 
        u"name": u"Belize", 
        u"continent": u"South America"
    }, 
    u"DYBJ": {
        u"region": u"Western Africa", 
        u"name": u"Dahomey", 
        u"continent": u"Africa"
    }, 
    u"IN-JK": {
        u"region": u"Southern Asia", 
        u"name": u"Jammu and Kashmir", 
        u"continent": u"Asia"
    }, 
    u"GG-SA": {
        u"region": u"Northern Europe", 
        u"name": u"Sark", 
        u"continent": u"Europe"
    }, 
    u"CY-NC": {
        u"region": u"Western Asia", 
        u"name": u"Northern Cyprus", 
        u"continent": u"Asia"
    }, 
    u"ML-AZ": {
        u"region": u"Western Africa", 
        u"name": u"Azawad", 
        u"continent": u"Africa"
    }, 
    u"OM": {
        u"region": u"Western Asia", 
        u"name": u"Oman", 
        u"continent": u"Asia"
    }, 
    u"DDDE": {
        u"region": u"Western Europe", 
        u"name": u"East Germany", 
        u"continent": u"Europe"
    }, 
    u"PCHH": {
        u"region": u"Micronesia", 
        u"name": u"Pacific Islands", 
        u"continent": u"Oceania"
    }, 
    u"HR": {
        u"region": u"Southern Europe", 
        u"name": u"Croatia", 
        u"continent": u"Europe"
    }, 
    u"AC": {
        u"region": u"Western Africa", 
        u"name": u"Ascension", 
        u"continent": u"Africa"
    }, 
    u"HT": {
        u"region": u"Caribbean", 
        u"name": u"Haiti", 
        u"continent": u"South America"
    }, 
    u"FQHH": {
        u"region": u"Antarctica", 
        u"name": u"French Southern and Antarctic Territories", 
        u"continent": u"Antarctica"
    }, 
    u"HK": {
        u"region": u"Eastern Asia", 
        u"name": u"Hong Kong", 
        u"continent": u"Asia"
    }, 
    u"HN": {
        u"region": u"Central America", 
        u"name": u"Honduras", 
        u"continent": u"South America"
    }, 
    u"HM": {
        u"region": u"Antarctica", 
        u"name": u"Heard Island and McDonald Islands", 
        u"continent": u"Antarctica"
    }, 
    u"PUUM": {
        u"region": u"Polynesia", 
        u"name": u"United States Miscellaneous Pacific Islands", 
        u"continent": u"Oceania"
    }, 
    u"GETV": {
        u"region": u"Polynesia", 
        u"name": u"Ellice Islands", 
        u"continent": u"Oceania"
    }, 
    u"ZA-VE": {
        u"region": u"Southern Africa", 
        u"name": u"Venda", 
        u"continent": u"Africa"
    }, 
    u"GBAE": {
        u"region": u"Western Asia", 
        u"name": u"Trucial States", 
        u"continent": u"Asia"
    }, 
    u"KHKA": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Khmer Republic", 
        u"continent": u"Asia"
    }, 
    u"UY": {
        u"region": u"Southern America", 
        u"name": u"Uruguay", 
        u"continent": u"South America"
    }, 
    u"UZ": {
        u"region": u"Central Asia", 
        u"name": u"Uzbekistan", 
        u"continent": u"Asia"
    }, 
    u"US": {
        u"region": u"Northern America", 
        u"aliases": [
            u"USA"
        ], 
        u"name": u"United States", 
        u"continent": u"North America"
    }, 
    u"UM": {
        u"region": u"Polynesia", 
        u"name": u"United States Minor Outlying Islands", 
        u"continent": u"Oceania"
    }, 
    u"UK": {
        u"region": u"Northern Europe", 
        u"aliases": [
            u"UK"
        ], 
        u"name": u"United Kingdom", 
        u"continent": u"Europe"
    }, 
    u"AU": {
        u"region": u"Australia and New Zealand", 
        u"name": u"Australia", 
        u"continent": u"Oceania"
    }, 
    u"UG": {
        u"region": u"Eastern Africa", 
        u"name": u"Uganda", 
        u"continent": u"Africa"
    }, 
    u"UA": {
        u"region": u"Eastern Europe", 
        u"name": u"Ukraine", 
        u"continent": u"Europe"
    }, 
    u"RHZW-RH": {
        u"region": u"Eastern Africa", 
        u"name": u"Rhodesia", 
        u"continent": u"Africa"
    }, 
    u"NI": {
        u"region": u"Central America", 
        u"name": u"Nicaragua", 
        u"continent": u"South America"
    }, 
    u"NL": {
        u"region": u"Western Europe", 
        u"name": u"Netherlands", 
        u"continent": u"Europe"
    }, 
    u"NO": {
        u"region": u"Northern Europe", 
        u"name": u"Norway", 
        u"continent": u"Europe"
    }, 
    u"NA": {
        u"region": u"Southern Africa", 
        u"name": u"Namibia", 
        u"continent": u"Africa"
    }, 
    u"NC": {
        u"region": u"Melanesia", 
        u"name": u"New Caledonia", 
        u"continent": u"Oceania"
    }, 
    u"NE": {
        u"region": u"Western Africa", 
        u"name": u"Niger", 
        u"continent": u"Africa"
    }, 
    u"NF": {
        u"region": u"Australia and New Zealand", 
        u"name": u"Norfolk Island", 
        u"continent": u"Oceania"
    }, 
    u"NG": {
        u"region": u"Western Africa", 
        u"name": u"Nigeria", 
        u"continent": u"Africa"
    }, 
    u"NZ": {
        u"region": u"Australia and New Zealand", 
        u"name": u"New Zealand", 
        u"continent": u"Oceania"
    }, 
    u"NP": {
        u"region": u"Southern Asia", 
        u"name": u"Nepal", 
        u"continent": u"Asia"
    }, 
    u"AZ-NK": {
        u"region": u"Western Asia", 
        u"name": u"Nagorno-Karabakh", 
        u"continent": u"Asia"
    }, 
    u"NU": {
        u"region": u"Polynesia", 
        u"name": u"Niue", 
        u"continent": u"Oceania"
    }, 
    u"HU": {
        u"region": u"Eastern Europe", 
        u"name": u"Hungary", 
        u"continent": u"Europe"
    }, 
    u"RHZW": {
        u"region": u"Eastern Africa", 
        u"name": u"Southern Rhodesia", 
        u"continent": u"Africa"
    }, 
    u"AE-DU": {
        u"region": u"Western Asia", 
        u"name": u"Dubai", 
        u"continent": u"Asia"
    }, 
    u"GB-SCT": {
        u"region": u"Northern Europe", 
        u"name": u"Scotland", 
        u"continent": u"Europe"
    }, 
    u"TZ": {
        u"region": u"Eastern Africa", 
        u"name": u"Tanzania", 
        u"continent": u"Africa"
    }, 
    u"TV": {
        u"region": u"Polynesia", 
        u"name": u"Tuvalu", 
        u"continent": u"Oceania"
    }, 
    u"TW": {
        u"region": u"Eastern Asia", 
        u"name": u"Taiwan", 
        u"continent": u"Asia"
    }, 
    u"TT": {
        u"region": u"Caribbean", 
        u"name": u"Trinidad and Tobago", 
        u"continent": u"South America"
    }, 
    u"CL": {
        u"region": u"Southern America", 
        u"name": u"Chile", 
        u"continent": u"South America"
    }, 
    u"TR": {
        u"region": u"Western Asia", 
        u"name": u"Turkey", 
        u"continent": u"Asia"
    }, 
    u"TN": {
        u"region": u"Northern Africa", 
        u"name": u"Tunisia", 
        u"continent": u"Africa"
    }, 
    u"TO": {
        u"region": u"Polynesia", 
        u"name": u"Tonga", 
        u"continent": u"Oceania"
    }, 
    u"TL": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Timor-Leste", 
        u"continent": u"Asia"
    }, 
    u"TM": {
        u"region": u"Central Asia", 
        u"name": u"Turkmenistan", 
        u"continent": u"Asia"
    }, 
    u"TJ": {
        u"region": u"Central Asia", 
        u"name": u"Tajikistan", 
        u"continent": u"Asia"
    }, 
    u"TK": {
        u"region": u"Polynesia", 
        u"name": u"Tokelau", 
        u"continent": u"Oceania"
    }, 
    u"TH": {
        u"region": u"South-Eastern Asia", 
        u"name": u"Thailand", 
        u"continent": u"Asia"
    }, 
    u"TF": {
        u"region": u"Antarctica", 
        u"name": u"French Southern Territories", 
        u"continent": u"Antarctica"
    }, 
    u"TG": {
        u"region": u"Western Africa", 
        u"name": u"Togo", 
        u"continent": u"Africa"
    }, 
    u"TD": {
        u"region": u"Middle Africa", 
        u"name": u"Chad", 
        u"continent": u"Africa"
    }, 
    u"TC": {
        u"region": u"Caribbean", 
        u"name": u"Turks and Caicos Islands", 
        u"continent": u"South America"
    }, 
    u"TA": {
        u"region": u"Western Africa", 
        u"name": u"Tristan da Cunha", 
        u"continent": u"Africa"
    }, 
    u"GE-SK": {
        u"region": u"Western Asia", 
        u"name": u"South Ossetia", 
        u"continent": u"Asia"
    }, 
    u"AE": {
        u"region": u"Western Asia", 
        u"name": u"United Arab Emirates", 
        u"continent": u"Asia"
    }, 
    u"AD": {
        u"region": u"Southern Europe", 
        u"name": u"Andorra", 
        u"continent": u"Europe"
    }, 
    u"AG": {
        u"region": u"Caribbean", 
        u"name": u"Antigua and Barbuda", 
        u"continent": u"South America"
    }, 
    u"AF": {
        u"region": u"Southern Asia", 
        u"name": u"Afghanistan", 
        u"continent": u"Asia"
    }, 
    u"AI": {
        u"region": u"Caribbean", 
        u"name": u"Anguilla", 
        u"continent": u"South America"
    }, 
    u"AM": {
        u"region": u"Western Asia", 
        u"name": u"Armenia", 
        u"continent": u"Asia"
    }, 
    u"AL": {
        u"region": u"Southern Europe", 
        u"name": u"Albania", 
        u"continent": u"Europe"
    }, 
    u"AO": {
        u"region": u"Middle Africa", 
        u"name": u"Angola", 
        u"continent": u"Africa"
    }, 
    u"AQ": {
        u"region": u"Antarctica", 
        u"name": u"Antarctica", 
        u"continent": u"Antarctica"
    }, 
    u"AS": {
        u"region": u"Polynesia", 
        u"name": u"American Samoa", 
        u"continent": u"Oceania"
    }, 
    u"AR": {
        u"region": u"Southern America", 
        u"name": u"Argentina", 
        u"continent": u"South America"
    }, 
    u"EGEG": {
        u"region": u"Northern Africa", 
        u"name": u"United Arab Republic", 
        u"continent": u"Africa"
    }, 
    u"AT": {
        u"region": u"Western Europe", 
        u"name": u"Austria", 
        u"continent": u"Europe"
    }, 
    u"AW": {
        u"region": u"Caribbean", 
        u"name": u"Aruba", 
        u"continent": u"South America"
    }, 
    u"AX": {
        u"region": u"Northern Europe", 
        u"name": u"Åland Islands", 
        u"continent": u"Europe"
    }, 
    u"AZ": {
        u"region": u"Western Asia", 
        u"name": u"Azerbaijan", 
        u"continent": u"Asia"
    }, 
    u"YDYE": {
        u"region": u"Western Asia", 
        u"name": u"South Yemen", 
        u"continent": u"Asia"
    }
}

# See http://en.wikipedia.org/wiki/WGS-84
EARTH_RADIUS = 6378137

def crosses_dateline(west, east):
    return west['lng'] > east['lng']

def get_area(southwest, northeast):
    def radians(point):
        return {
            'lat': math.radians(point['lat']),
            'lng': math.radians(point['lng'])
        }
    if crosses_dateline(southwest, northeast):
        northeast['lng'] += 360
    southwest = radians(southwest)
    northeast = radians(northeast)
    return math.pow(EARTH_RADIUS, 2) * abs(
        math.sin(southwest['lat']) - math.sin(northeast['lat'])
    ) * abs(southwest['lng'] - northeast['lng']);

def get_country(code_or_name):
    if isinstance(code_or_name, bytes):
        code_or_name = code_or_name.decode('utf-8')
    if len(code_or_name) == 2:
        code_or_name = code_or_name.upper()
        return COUNTRIES[code_or_name] if code_or_name in COUNTRIES else {}
    else:
        for code, country in COUNTRIES.items():
            if code_or_name == country['name'] or (
                'aliases' in country and code_or_name in country['aliases']
            ):
                return country
    return {}

def get_country_name(country_code):
    country_code = country_code.upper()
    return COUNTRIES[country_code]['name'] if country_code in COUNTRIES else ''

def normalize_country_name(country_name):
    if isinstance(country_name, bytes):
        country_name = country_name.decode('utf-8')
    name = None
    for code, country in COUNTRIES.items():
        if country_name == country['name'] or (
            'aliases' in country and country_name in country['aliases']
        ):
            name = country['name']
            break
    return name

def split_geoname(geoname):
    if isinstance(geoname, bytes):
        geoname = geoname.decode('utf-8')
    countries = [
        u'Bonaire, Sint Eustatius and Saba',
        u'Saint Helena, Ascension and Tristan da Cunha'
    ]
    for country in countries:
        if geoname.endswith(country):
            geoname = geoname.replace(country, country.replace(', ', '; '))
    split = geoname.split(', ')
    for country in countries:
        if geoname.endswith(country.replace(', ', '; ')):
            split[-1] = country
    return split



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from datetime import datetime
import re

from urllib.parse import quote

from ox import find_re, cache, strip_tags, decode_html, get_torrent_info, normalize_newlines
from ox.normalize import normalize_imdbid
import ox

cache_timeout = 24*60*60 # cache search only for 24 hours

season_episode = re.compile("S..E..", re.IGNORECASE)
baseurl = "https://thepiratebay.org/"


def read_url(url, data=None, headers=cache.DEFAULT_HEADERS, timeout=cache.cache_timeout, valid=None, unicode=False):
    headers = headers.copy()
    headers['Cookie'] = 'language=en_EN'
    return cache.read_url(url, data, headers, timeout, unicode=unicode)

def find_movies(query=None, imdb=None, max_results=10):
    if imdb:
        query = "tt" + normalize_imdbid(imdb)
    results = []
    next = [baseurl + "hsearch/%s/0/3/200" % quote(query), ]
    page_count = 1
    while next and page_count < 4:
        page_count += 1
        url = next[0]
        if not url.startswith('http'):
            if not url.startswith('/'):
                url = "/" + url
            url = baseurl + url
        data = read_url(url, timeout=cache_timeout, unicode=True)
        regexp = '''<tr.*?<td class="vertTh"><a href="/browse/(.*?)".*?<td><a href="(/torrent/.*?)" class="detLink".*?>(.*?)</a>.*?</tr>'''
        for row in  re.compile(regexp, re.DOTALL).findall(data):
            torrentType = row[0]
            torrentLink = baseurl + row[1]
            torrentTitle = decode_html(row[2])
            # 201 = Movies , 202 = Movie DVDR, 205 TV Shows
            if torrentType in ['201']:
                results.append((torrentTitle, torrentLink, ''))
            if len(results) >= max_results:
                return results
        next = re.compile('<a.*?href="(.*?)".*?>.*?next.gif.*?</a>').findall(data)
    return results

def get_id(piratebayId):
    if piratebayId.startswith('http://torrents.thepiratebay.org/'):
        piratebayId = piratebayId.split('org/')[1]
    d = find_re(piratebayId, "tor/(\d+)")
    if d:
        piratebayId = d
    d = find_re(piratebayId, "torrent/(\d+)")
    if d:
        piratebayId = d
    return piratebayId

def exists(piratebayId):
    piratebayId = get_id(piratebayId)
    return ox.net.exists(baseurl + "torrent/%s" % piratebayId)

def get_data(piratebayId):
    _key_map = {
      'spoken language(s)': u'language',
      'texted language(s)': u'subtitle language',
      'by': u'uploader',
      'leechers': 'leecher',
      'seeders': 'seeder',
    }
    piratebayId = get_id(piratebayId)
    torrent = dict()
    torrent[u'id'] = piratebayId
    torrent[u'domain'] = 'thepiratebay.org'
    torrent[u'comment_link'] = baseurl + 'torrent/%s' % piratebayId

    data = read_url(torrent['comment_link'], unicode=True)
    torrent[u'title'] = find_re(data, '<title>(.*?) \(download torrent\) - TPB</title>')
    if not torrent[u'title']:
        return None
    torrent[u'title'] = decode_html(torrent[u'title']).strip()
    torrent[u'imdbId'] = find_re(data, 'title/tt(\d{7})')
    title = quote(torrent['title'].encode('utf-8'))
    torrent[u'magent_link']= find_re(data, '"(magnet:.*?)"')
    torrent[u'infohash'] = find_re(torrent[u'magent_link'], "btih:(.*?)&")
    for d in re.compile('dt>(.*?):</dt>.*?<dd.*?>(.*?)</dd>', re.DOTALL).findall(data):
        key = d[0].lower().strip()
        key = _key_map.get(key, key)
        value = decode_html(strip_tags(d[1].strip()))
        if not '<' in key:
            torrent[key] = value
    torrent[u'description'] = find_re(data, '<div class="nfo">(.*?)</div>')
    if torrent[u'description']:
        torrent['description'] = normalize_newlines(decode_html(strip_tags(torrent['description']))).strip()
    return torrent



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
# GPL 2008
import gzip
import math
import re
import unicodedata
from io import BytesIO

from functools import reduce

ARTICLES = list(set([
    # def sg, def pl, indef sg, indef pl (each m/f/n)
    'der', 'die', 'das', 'ein', 'eine',  # de
    'the', 'a', 'an',  # en
    'el', 'la', 'lo', 'los', 'las', 'un', 'una', 'unos', 'unas',  # es
    'le', "l'", 'la', 'les', 'un', 'une', 'des',  # fr
    'il', 'lo', "l'" 'la', '_i', 'gli', 'le',  # it
    'de', 'het', 'een',  # nl
    'o', 'a', 'os', '_as', 'um', 'uma', '_uns', 'umas'  # pt
    # some _disabled because of collisions
]))
# every given name in 0xDB that matches Xxxx-yyyy Lastname
ASIAN_FIRST_NAMES = [
    'a', 'ae', 'aeng', 'ah', 'ai', 'an', 'back', 'bae', 'ban', 'bang', 'bao', 
    'beom', 'bi', 'bin', 'bo', 'bok', 'bon', 'bong', 'bu', 'bum', 'byeong', 
    'byoung', 'byung', 'cai', 'chae', 'chan', 'chang', 'chao', 'cheal', 'chen', 
    'cheng', 'cheol', 'cheon', 'cheong', 'cheul', 'chi', 'chia', 'chiao', 
    'chieh', 'chien', 'chih', 'chin', 'ching', 'cho', 'choi', 'chong', 'choo', 
    'chu', 'chuan', 'chuen', 'chul', 'chun', 'chung', 'chuo', 'chyi', 'da', 
    'dae', 'dah', 'dal', 'dan', 'deok', 'do', 'dong', 'doo', 'duek', 'duk', 
    'e', 'el', 'en', 'eui', 'eul', 'eun', 'eung', 'fai', 'fan', 'fang', 'fei', 
    'fen', 'feng', 'fo', 'foo', 'fu', 'ga', 'gae', 'gam', 'gang', 'ge', 'gen', 
    'geon', 'geun', 'gi', 'gil', 'gin', 'gnad', 'gok', 'goo', 'gook', 'gu', 
    'gun', 'gwan', 'gye', 'gyeong', 'gyu', 'gyun', 'ha', 'hae', 'hak', 'han', 
    'hang', 'hao', 'he', 'hee', 'heng', 'heon', 'hie', 'ho', 'hoi', 'hong', 
    'hoo', 'hoon', 'hou', 'hsi', 'hsiang', 'hsiao', 'hsieh', 'hsien', 'hsin', 
    'hsing', 'hsiung', 'hu', 'hua', 'huai', 'huang', 'hue', 'hui', 'hun', 
    'hung', 'hwa', 'hwan', 'hwang', 'hye', 'hyeok', 'hyeon', 'hyeong', 'hyo', 
    'hyuk', 'hyun', 'hyung', 'i', 'ik', 'il', 'in', 'ja', 'jae', 'jan', 'jang', 
    'je', 'jee', 'jen', 'jeok', 'jeong', 'jeung', 'ji', 'jia', 'jian', 'jik', 
    'jin', 'jing', 'jo', 'jong', 'joo', 'joon', 'ju', 'juan', 'jun', 'jung', 
    'ka', 'kai', 'kam', 'kan', 'kang', 'kap', 'kar', 'ke', 'kee', 'kei', 
    'keng', 'keum', 'keung', 'ki', 'kil', 'kin', 'kit', 'kot', 'ku', 'kua', 
    'kuan', 'kuang', 'kuen', 'kun', 'kuo', 'kwang', 'kwok', 'kwon', 'kwong', 
    'kyeong', 'kyo', 'kyoon', 'kyou', 'kyoung', 'kyu', 'kyun', 'kyung', 'lai', 
    'lau', 'lee', 'lei', 'leng', 'leung', 'li', 'liang', 'lien', 'lin', 'ling', 
    'lock', 'long', 'lun', 'lung', 'maeng', 'man', 'mei', 'mi', 'miao', 'min', 
    'ming', 'mo', 'mok', 'moo', 'mook', 'moon', 'mu', 'mun', 'myeong', 
    'myoeng', 'myong', 'myung', 'na', 'nae', 'nai', 'nam', 'nan', 'neung', 
    'ngaru', 'ni', 'no', 'nyeo', 'oh', 'ok', 'ou', 'pai', 'pei', 'pen', 'peng', 
    'pi', 'pil', 'pin', 'ping', 'po', 'pui', 'pyo', 'pyung', 'qing', 'qun', 
    'ra', 'rak', 'ram', 'ran', 'reum', 'ri', 'rim', 'rin', 'roe', 'rok', 'ru', 
    'rui', 'ryeon', 'ryol', 'ryong', 'sa', 'sae', 'san', 'sang', 'se', 'seo', 
    'seob', 'seok', 'seol', 'seon', 'seong', 'seung', 'shan', 'shen', 'sheng', 
    'shi', 'shia', 'shiang', 'shih', 'shik', 'shim', 'shin', 'shing', 'shou', 
    'shu', 'shun', 'si', 'sik', 'sin', 'siu', 'so', 'song', 'soo', 'sook', 
    'soon', 'su', 'suk', 'sun', 'sung', 'sup', 'szu', "t'ien", 'ta', 'tae', 
    'taek', 'tai', 'tak', 'te', 'ti', 'tian', 'ting', 'to', 'toa', 'tsai', 
    'tsan', 'tse', 'tso', 'tsui', 'tung', 'tzu', 'ua', 'ui', 'un', 'wah', 
    'wai', 'wan', 'wei', 'wen', 'weon', 'wing', 'wit', 'wol', 'won', 'woo', 
    'wook', 'woon', 'woong', 'wuk', 'xiao', 'ya', 'yan', 'yang', 'yao', 'ye', 
    'yea', 'yee', 'yeh', 'yen', 'yeo', 'yeol', 'yeon', 'yeong', 'yeop', 'yi', 
    'yin', 'ying', 'yiu', 'yoeng', 'yong', 'yoo', 'yoon', 'you', 'young', 'yu', 
    'yuan', 'yue', 'yuen', 'yuk', 'yull', 'yun', 'yune', 'yung', 'zhi', 
    'zhong', 'zhu'
]
# see http://en.wikipedia.org/wiki/List_of_common_Chinese_surnames
# and http://en.wikipedia.org/wiki/List_of_Korean_family_names
ASIAN_LAST_NAMES = [
    'chan', 'chang', 'chao',
    'chen', 'cheong', 'cheung',
    'chong', 'choo',
    'chu', 'chun',
    'hou', 'hsieh', 'hsu', 'hu', 'huang',
    'kuo',
    'li', 'liang', 'lin', 'liu',
    '_park',
    'sun', 'sung',
    'tsao',
    'wang', 'Wong',
    'yang', 'yeong', 'yeung'
]
PREFIXES = [
    'al', 'bin', 'da', 'de', 'del', 'dem', 'den', 'der', 'di', 'dos', 'du',
    'e', 'el', 'la', 'san', 'the', 'van', 'vom', 'von', 'y', 'zu'
]
MIDFIXES = ['und']
SUFFIXES = ['ii', 'iii', 'jr', 'jr.', 'ph.d.', 'phd', 'sr', 'sr.']

UA_ALIASES = {
    'browser': {
        'Chrome': '(CriOS|CrMo)',
        'Firefox': '(Fennec|Firebird|Iceweasel|Minefield|Namoroka|Phoenix|SeaMonkey|Shiretoko)',
        'Nokia Browser': '(OviBrowser)'
    },
    'robot': {},
    'system': {
        'BSD': '(FreeBSD|NetBSD|OpenBSD)',
        'Linux': '(CrOS|MeeGo|webOS)',
        'Unix': '(AIX|HP-UX|IRIX|SunOS)'
    }
}
UA_NAMES = {
    'browser': {
        'chromeframe': 'Chrome Frame',
        'FBForIPhone': 'WebKit',
        'Gecko': 'Mozilla',
        'IEMobile': 'Internet Explorer',
        'konqueror': 'Konqueror',
        'Mozilla': 'Netscape',
        'MSIE': 'Internet Explorer',
        'NokiaBrowser': 'Nokia Browser',
        'Trident': 'Internet Explorer'
    },
    'robot': {},
    'system': {
        'BB': 'BlackBerry',
        'CPU OS': 'iOS',
        'iPhone': 'iOS',
        'iPhone OS': 'iOS',
        'J2ME/MIDP': 'Java',
        'Mac_PowerPC': 'Mac OS',
        'Mac_PPC': 'Mac OS',
        'Macintosh': 'Mac OS',
        'PLAYSTATION': 'PlayStation',
        'S': 'Nokia',
        'Series': 'Nokia',
        'Win': 'Windows',
        'Windows Phone OS': 'Windows Phone',
        'X11': 'Linux'
    }
}
UA_REGEXPS = {
    'browser': [
        '(Camino)\/(\d+)',
        '(Chimera)\/(\d+)',
        '(chromeframe)\/(\d+)',
        '(Edge)\/(\d+)',
        '(Epiphany)\/(\d+)',  # before Chrome, Chromium and Safari
        '(Chromium)\/(\d+)',  # before Chrome
        '(Chrome)\/(\d+)',
        '(FBForIPhone)',
        '(Firefox)\/(\d+)',
        '(Galeon)\/(\d+)',
        '(IEMobile)\/(\d+)',
        '(iCab) (\d+)',
        '(iCab)\/(\d+)',
        '(konqueror)\/(\d+)',
        '(Konqueror)\/(\d+)',
        '(Lynx)\/(\d+)',
        '(Netscape)\d?\/(\d+)',
        '(NokiaBrowser)\/(\d+)',
        '(OmniWeb)\/(\d+)',
        '(Opera)\/.+Version\/(\d+)',
        '(OviBrowser)\/(\d+)',
        'Version\/(\d+).+(Safari)',
        '(WebKit)\/(\d+)',
        '(MSIE) (\d\d?(?!\d))',  # last, since Opera used to mask as MSIE
        '(Trident)\/.*?rv:(\d+)',
        '(Gecko)',
        '(Mozilla)\/(3|4)'
    ],
    'robot': [
        '(BingPreview)\/(\d+)',
        '(Google Web Preview).+Chrome\/(\d+)',
        '(Googlebot)\/(\d+)',
        '(WebCrawler)\/(\d+)',
        '(Yahoo! Slurp)\/(\d+)',
        '(YandexBot)\/([\d\.]+)',
        '(YandexMobileBot)\/([\d\.]+)',
    ],
    'system': [
        '(Android) (\d+)',
        '(Android)',
        '(BB)(\d+)',
        '(BeOS)',
        '(BlackBerry) (\d+)',
        '(BlackBerry)',
        '(Darwin)',
        '(BSD) (FreeBSD|NetBSD|OpenBSD)',
        '(CPU OS) (\d+)',
        '(iPhone OS) (\d+)',
        '(iPhone)',  # Opera
        '(J2ME\/MIDP)',
        '(Linux).+(CentOS|CrOS|Debian|Fedora|Gentoo|Mandriva|MeeGo|Mint|Red Hat|SUSE|Ubuntu|webOS)',
        '(CentOS|CrOS|Debian|Fedora|Gentoo|Mandriva|MeeGo|Mint|Red Hat|SUSE|Ubuntu|webOS).+(Linux)',
        '(Linux)',
        '(Mac OS X) (10.\d+)',
        '(Mac OS X)',
        '(Mac_PowerPC)',
        '(Mac_PPC)',
        '(Macintosh)',
        'Nintendo (Wii).+NX\/(\d+)',
        '(PLAYSTATION) (\d+)',
        '(PlayStation) Vita (\d+)',
        '(RIM Tablet OS) (\d+)',
        '(S)(60);',
        '(Series) ?(40|60)',
        '(Symbian OS)',
        '(SymbianOS)\/(\d+)',
        '(SymbOS)',
        '(OS\/2)',
        '(Unix) (AIX|HP-UX|IRIX|SunOS)',
        '(Unix)',
        '(Windows) (NT \d\.\d)',
        '(Windows Phone) (\d+)',
        '(Windows Phone OS) (\d+)',
        '(Windows) (3\.1|95|98|2000|2003|CE|ME|Mobile|NT|XP)',  # Opera
        '(Win) (9x 4\.90)',  # Firefox
        '(Win)(16)',  # Firefox
        '(Win)(9\d)',  # Firefox
        '(Win)(NT)',  # Firefox
        '(Win)(NT4\.0)',  # Firefox
        '(X11)'
    ]
}
UA_VERSIONS = {
    'browser': {},
    'robot': {},
    'system': {
        '10.0': '10.0 (Cheetah)',
        '10.1': '10.1 (Puma)',
        '10.2': '10.2 (Jaguar)',
        '10.3': '10.3 (Panther)',
        '10.4': '10.4 (Tiger)',
        '10.5': '10.5 (Leopard)',
        '10.6': '10.6 (Snow Leopard)',
        '10.7': '10.7 (Lion)',
        '10.8': '10.8 (Mountain Lion)',
        '10.9': '10.9 (Mavericks)',
        '10.10': '10.10 (Yosemite)',
        '10.11': '10.11 (El Capitan)',
        '40': 'Series 40',
        '60': 'Series 60',
        'NT 3.1': 'NT 3.1 (3.1)',
        'NT 3.5': 'NT 3.5 (NT)',
        'NT 4.0': 'NT 4.0 (NT)',
        'NT 4.1': 'NT 4.1 (98)',
        '9x 4.90': 'NT 4.9 (ME)',
        'NT 5.0': 'NT 5.0 (2000)',
        'NT 5.1': 'NT 5.1 (XP)',
        'NT 5.2': 'NT 5.2 (2003)',
        'NT 6.0': 'NT 6.0 (Vista)',
        'NT 6.1': 'NT 6.1 (7)',
        'NT 6.2': 'NT 6.2 (8)',
        'NT 6.3': 'NT 6.3 (8.1)',
        'NT 6.4': 'NT 6.4 (10)',
        '16': 'NT 3.1 (3.1)',
        '3.1': 'NT 3.1 (3.1)',
        '95': 'NT 4.0 (95)',
        'NT': 'NT 4.0 (NT)',
        'NT4.0': 'NT 4.0 (NT)',
        '98': 'NT 4.1 (98)',
        'ME': 'NT 4.9 (ME)',
        '2000': 'NT 5.0 (2000)',
        'XP': 'NT 5.1 (XP)',
        '2003': 'NT 5.2 (2003)'
    }
}

def get_sort_name(name):
    """

    >>> get_sort_name('Alfred Hitchcock')
    'Hitchcock, Alfred'

    >>> get_sort_name('Jean-Luc Godard')
    'Godard, Jean-Luc'

    >>> get_sort_name('Rainer Werner Fassbinder')
    'Fassbinder, Rainer Werner'

    >>> get_sort_name('Brian De Palma')
    'De Palma, Brian'

    >>> get_sort_name('Johan van der Keuken')
    'van der Keuken, Johan'

    >>> get_sort_name('Edward D. Wood Jr.')
    'Wood Jr., Edward D.'

    >>> get_sort_name('Bing Wang')
    'Wang Bing'

    >>> get_sort_name('Frank Capra III')
    'Capra III, Frank'

    >>> get_sort_name('The Queen of England')
    'Queen of England, The'

    >>> get_sort_name('Sham 69')
    'Sham 69'

    >>> get_sort_name('Scorsese, Martin')
    'Scorsese, Martin'
    """
    if ' ' not in name or ', ' in name:
        return name
    if name.lower().startswith('the '):
        return get_sort_title(name)

    def add_name():
        if len(first_names):
            last_names.insert(0, first_names.pop())

    def find_name(names):
        return len(first_names) and first_names[-1].lower() in names

    if is_asian_name(name):
        names = name.replace('-', ' ').split(' ')
        if len(names) == 2:
            if names[0].lower() in ASIAN_LAST_NAMES:
                lastname, firstname = names
            else:
                firstname, lastname = names
        else:
            names_ = name.split(' ')
            if '-' in names_[0]:
                lastname, firstname = [names[2], names[0] + '-' + names[1].lower()]
            elif '-' in names_[1]:
                lastname, firstname = [names[0], names[1] + '-' + names[2].lower()]
            elif names[0].lower() in ASIAN_FIRST_NAMES and names[2].lower() not in ASIAN_FIRST_NAMES:
                lastname, firstname = [names[2], names[0] + ' ' + names[1]]
            elif names[0].lower() not in ASIAN_FIRST_NAMES and names[2].lower() in ASIAN_FIRST_NAMES:
                lastname, firstname = [names[0], names[1] + ' ' + names[2]]
            elif names[0].lower() in ASIAN_LAST_NAMES:
                lastname, firstname = [names[0], names[1] + ' ' + names[2]]
            else:
                lastname, firstname = [names[2], names[0] + ' ' + names[1]]
        return lastname + ' ' + firstname

    first_names = name.split(' ')
    last_names = []
    if re.search('^[0-9]+$', first_names[-1]):
        add_name()
    if re.search('[(\[].+?[)\]]$', first_names[-1]):
        add_name()
    if find_name(SUFFIXES):
        add_name()
    add_name()
    if find_name(MIDFIXES):
        add_name()
        add_name()
    while find_name(PREFIXES):
        add_name()
    name = ' '.join(last_names)
    if len(first_names):
        separator = ' ' if last_names[0].lower() in ASIAN_LAST_NAMES else ', '
        name += separator + ' '.join(first_names)
    return name

def get_sort_title(title):
    """

    >>> get_sort_title('Themroc')
    'Themroc'

    >>> get_sort_title('Die Hard')
    'Hard, Die'

    >>> get_sort_title("L'atalante")
    "atalante, L'"

    """
    for article in ARTICLES:
        spaces = 0 if article.endswith("'") else 1
        if title.lower().startswith(article + ' ' * spaces):
            length = len(article)
            return title[length + spaces:] + ', ' + title[:length]
    return title

def find_re(string, regexp):
    result = re.compile(regexp, re.DOTALL).findall(string)
    if result:
        return result[0].strip()
    return ''

def find_string(string, string0='', string1=''):
    """Return the string between string0 and string1.

    If string0 or string1 is left out, begining or end of string is used.

    >>> find_string('i am not there', string1=' not there')
    'i am'

    >>> find_string('i am not there', 'i am ', ' there')
    'not'

    >>> find_string('i am not there', 'i am not t')
    'here'

    """
    if string0:
        string0 = re.escape(string0)
    else:
        string0 = '^'
    if string1:
        string1 = re.escape(string1)
    else:
        string1 = '$'
    return find_re(string, string0 + '(.*?)' + string1)

def is_asian_name(name):
    names = name.replace('-', ' ').lower().split(' ')
    return (len(names) == 2 and not '-' in name and (
        (names[0] in ASIAN_FIRST_NAMES and names[1] in ASIAN_LAST_NAMES) or
        (names[0] in ASIAN_LAST_NAMES and names[1] in ASIAN_FIRST_NAMES)
    )) or (
        len(names) == 3 and names[1] in ASIAN_FIRST_NAMES and (
            names[0] in ASIAN_FIRST_NAMES or names[2] in ASIAN_FIRST_NAMES
        )
    )

def parse_useragent(useragent):
    data = {}
    for key in UA_REGEXPS:
        for alias, regexp in UA_ALIASES[key].items():
            alias = alias if key == 'browser' else alias + ' \\1'
            useragent = re.sub(regexp, alias, useragent)
        for regexp in UA_REGEXPS[key]:
            data[key] = {'name': '', 'version': '', 'string': ''}
            match = re.compile(regexp).search(useragent)
            if match:
                matches = list(match.groups())
                if len(matches) == 1:
                    matches.append('')
                swap = re.match('^\d', matches[0]) or matches[1] == 'Linux'
                name = matches[1 if swap else 0]
                version = matches[0 if swap else 1].replace('_', '.')
                name = UA_NAMES[key][name] if name in UA_NAMES[key] else name
                version = UA_VERSIONS[key][version] if version in UA_VERSIONS[key] else version
                string = name
                if version:
                    string = string + ' ' + (
                        '(' + version + ')' if name in ['BSD', 'Linux', 'Unix'] else version
                    )
                data[key] = {
                    'name': name,
                    'version': version,
                    'string': string
                }
                break
    return data

def remove_special_characters(text):
    """
    Removes special characters inserted by Word.
    """
    text = text.replace(u'\u2013', '-')
    text = text.replace(u'\u2026O', "'")
    text = text.replace(u'\u2019', "'")
    text = text.replace(u'', "'")
    text = text.replace(u'', "'")
    text = text.replace(u'', "-")
    return text

def wrap(text, width):
    """
    A word-wrap function that preserves existing line breaks and most spaces in
    the text. Expects that existing line breaks are posix newlines (\n).
    See http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/148061
    """

    def reduce_line(line, word):
        return '%s%s%s' % (
            line,
            ' \n'[
                (len(line[line.rfind('\n')+1:]) + len(word.split('\n', 1)[0]) >= width)
            ],
            word
        )

    return reduce(reduce_line, text.split(' '))

def wrap_string(string, length=80, separator='\n', balance=False):
    '''
    >>> wrap_string("Anticonstitutionellement, Paris s'eveille", 16)
    "Anticonstitution\\nellement, Paris \\ns'eveille"
    >>> wrap_string(u'All you can eat', 12, '\\n', True)
    'All you \\ncan eat'
    '''
    words = string.split(' ')
    if balance:
        # balance lines: test if same number of lines
        # can be achieved with a shorter line length
        lines = wrap_string(string, length, separator, False).split(separator)
        if len(lines) > 1:
            while length > max([len(x) for x in words]):
                length -= 1
                if len(wrap_string(string, length, separator, False).split(separator)) > len(lines):
                    length += 1
                    break
    lines = ['']
    for word in words:
        if len(lines[len(lines) - 1] + word + ' ') <= length + 1:
            # word fits in current line
            lines[len(lines) - 1] += word + ' '
        else:
            if len(word) <= length:
                # word fits in next line
                lines.append(word + ' ')
            else:
                # word is longer than line
                position = length - len(lines[len(lines) - 1])
                lines[len(lines) - 1] += word[0:position]
                for i in range(position, len(word), length):
                    lines.append(word[i:i+length])
                lines[len(lines) - 1] += ' '
    return separator.join(lines).strip()

def truncate_string(string, length, padding='...', position='right'):
    #  >>> truncate_string('anticonstitutionellement', 16, '...', 'left')
    #  '...utionellement'
    #  >>> truncate_string('anticonstitutionellement', 16, '...', 'center')
    #  'anticon...lement'
    #  >>> truncate_string('anticonstitutionellement', 16, '...', 'right')
    #  'anticonstitut...'
    stringLength = len(string)
    paddingLength = len(padding)
    if stringLength > length:
        if position == 'left':
            string = '%s%s' % (padding, string[stringLength + paddingLength - length:])
        elif position == 'center':
            left = int(math.ceil(float(length - paddingLength) / 2))
            right = int(stringLength - math.floor(float(length - paddingLength) / 2))
            string = '%s%s%s' % (string[:left], padding, string[right:])
        elif position == 'right':
            string = '%s%s' % (string[:length - paddingLength], padding)
    return string

def truncate_words(s, num):
    """Truncates a string after a certain number of chacters, but ends with a word

    >>> truncate_words('Truncates a string after a certain number of chacters, but ends with a word', 23)
    'Truncates a string...'
    >>> truncate_words('Truncates a string', 23)
    'Truncates a string'

    """
    length = int(num)
    if len(s) <= length:
        return s
    words = s.split()
    ts = ""
    while words and len(ts) + len(words[0]) < length:
        ts += " " + words.pop(0)
    if words:
        ts += "..."
    return ts.strip()

def trim_string(string, num):
    """Truncates a string after a certain number of chacters, adding ... at -10 characters

    >>> trim_string('Truncates a string after a certain number of chacters', 23)
    'Truncates ...f chacters'
    >>> trim_string('Truncates a string', 23)
    'Truncates a string'
    """
    if len(string) > num:
        string = string[:num - 13] + '...' + string[-10:]
    return string

def get_valid_filename(s):
    """
    Returns the given string converted to a string that can be used for a clean
    filename. Specifically, leading and trailing spaces are removed;
    all non-filename-safe characters are removed.

    >>> get_valid_filename("john's portrait in 2004.jpg")
    'john_s_portrait_in_2004.jpg'
    """
    s = s.strip()
    s = s.replace(' ', '_')
    s = re.sub(r'[^-A-Za-z0-9_.\[\]\ ]', '_', s)
    s = s.replace('__', '_').replace('__', '_')
    return s

def get_text_list(list_, last_word='or'):
    """
    >>> get_text_list(['a', 'b', 'c', 'd'])
    'a, b, c or d'
    >>> get_text_list(['a', 'b', 'c'], 'and')
    'a, b and c'
    >>> get_text_list(['a', 'b'], 'and')
    'a and b'
    >>> get_text_list(['a'])
    'a'
    >>> get_text_list([])
    ''
    """
    if len(list_) == 0:
        return ''
    if len(list_) == 1:
        return list_[0]
    return '%s %s %s' % (', '.join([i for i in list_][:-1]), last_word, list_[-1])

def get_list_text(text, last_word='or'):
    """
    >>> get_list_text('a, b, c or d')
    ['a', 'b', 'c', 'd']
    >>> get_list_text('a, b and c', 'and')
    ['a', 'b', 'c']
    >>> get_list_text('a and b', 'and')
    ['a', 'b']
    >>> get_list_text('a')
    ['a']
    >>> get_list_text('')
    []
    """
    list_ = []
    if text:
        list_ = text.split(', ')
        if list_:
            i = len(list_)-1
            last = list_[i].split(last_word)
            if len(last) == 2:
                list_[i] = last[0].strip()
                list_.append(last[1].strip())
    return list_

def normalize_newlines(text):
    return re.sub(r'\r\n|\r|\n', '\n', text)

def recapitalize(text):
    "Recapitalizes text, placing caps after end-of-sentence punctuation."
    # capwords = ()
    text = text.lower()
    capsRE = re.compile(r'(?:^|(?<=[\.\?\!] ))([a-z])')
    text = capsRE.sub(lambda x: x.group(1).upper(), text)
    # for capword in capwords:
    #    capwordRE = re.compile(r'\b%s\b' % capword, re.I)
    #    text = capwordRE.sub(capword, text)
    return text

def phone2numeric(phone):
    "Converts a phone number with letters into its numeric equivalent."
    letters = re.compile(r'[A-PR-Y]', re.I)

    def char2number(m):
        return {
            'a': '2', 'c': '2', 'b': '2', 'e': '3',
            'd': '3', 'g': '4', 'f': '3', 'i': '4', 'h': '4', 'k': '5',
            'j': '5', 'm': '6', 'l': '5', 'o': '6', 'n': '6', 'p': '7',
            's': '7', 'r': '7', 'u': '8', 't': '8', 'w': '9', 'v': '8',
            'y': '9', 'x': '9'
        }.get(m.group(0).lower())
    return letters.sub(char2number, phone)

def compress_string(s):
    zbuf = BytesIO()
    zfile = gzip.GzipFile(mode='wb', compresslevel=6, fileobj=zbuf)
    zfile.write(s)
    zfile.close()
    return zbuf.getvalue()

smart_split_re = re.compile('("(?:[^"\\\\]*(?:\\\\.[^"\\\\]*)*)"|\'(?:[^\'\\\\]*(?:\\\\.[^\'\\\\]*)*)\'|[^\\s]+)')

def smart_split(text):
    """
    Generator that splits a string by spaces, leaving quoted phrases together.
    Supports both single and double quotes, and supports escaping quotes with
    backslashes. In the output, strings will keep their initial and trailing
    quote marks.
    >>> list(smart_split('This is "a person\\'s" test.'))
    ['This', 'is', '"a person\\'s"', 'test.']
    """
    for bit in smart_split_re.finditer(text):
        bit = bit.group(0)
        if bit[0] == '"':
            yield '"' + bit[1:-1].replace('\\"', '"').replace('\\\\', '\\') + '"'
        elif bit[0] == "'":
            yield "'" + bit[1:-1].replace("\\'", "'").replace("\\\\", "\\") + "'"
        else:
            yield bit

def words(text):
    """
        returns words in text, removing punctuation
    """
    text = text.split()
    return [re.sub("(([.!?:-_]|'s)$)", '', x) for x in text]

def sort_string(string):
    string = string.replace('Æ', 'AE').replace('Ø', 'O').replace('Þ', 'Th')

    # pad numbered titles
    string = re.sub('(\d),(\d{3})', '\\1\\2', string)
    string = re.sub('(\d+)', lambda x: '%010d' % int(x.group(0)), string)
    return unicodedata.normalize('NFKD', string)

def sorted_strings(strings, key=None):
    if not key:
        key = sort_string
    return sorted(strings, key=key)



# -*- coding: UTF-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import print_function

import re

from ox.cache import read_url
from ox import find_re

def get_data(id):
    '''
    >>> get_data('0060304')['posters'][0]
    u'http://www.movieposterdb.com/posters/06_03/1967/0060304/l_99688_0060304_639fdd1e.jpg'
    >>> get_data('0123456')['posters']
    []
    '''
    data = {
        "url": get_url(id)
    }
    data["posters"] = get_posters(data["url"])
    return data

def get_id(url):
    return url.split("/")[-2]

def get_posters(url, group=True, timeout=-1):
    posters = []
    html = read_url(url, timeout=timeout, unicode=True)
    if url in html:
        if group:
            results = re.compile('<a href="(http://www.movieposterdb.com/group/.+?)\??">', re.DOTALL).findall(html)
            for result in results:
                posters += get_posters(result, False)
        results = re.compile('<a href="(http://www.movieposterdb.com/poster/.+?)">', re.DOTALL).findall(html)
        for result in results:
            html = read_url(result, timeout=timeout, unicode=True)
            posters.append(find_re(html, '"(http://www.movieposterdb.com/posters/.+?\.jpg)"'))
    return posters

def get_url(id):
    return "http://www.movieposterdb.com/movie/%s/" % id

if __name__ == '__main__':
    print(get_data('0060304'))
    print(get_data('0133093'))



#!/usr/bin/env python
# vi:si:et:sw=4:sts=4:ts=4
# encoding: utf-8

from setuptools import setup

def get_git_version():
    import subprocess
    version = subprocess.check_output(['git', 'describe', '--tags']).decode().strip().replace('-', '.')
    return '.'.join((version.split('.') + ['0'])[:3])

def get_version():
    return '3.0.10' #import os
    import re
    _git = os.path.join(os.path.dirname(__file__), '.git')
    __version = os.path.join(os.path.dirname(__file__), 'ox/__version.py')
    changelog = os.path.join(os.path.dirname(__file__), 'debian/changelog')
    if os.path.exists(_git):
        version = get_git_version()
        if version:
            with open(__version, 'w') as fd:
                fd.write('VERSION="%s"' % version)
            return version
    elif os.path.exists(__version):
        with open(__version) as fd:
            data = fd.read().strip().split('\n')[0]
            version = re.compile('VERSION="(.*)"').findall(data)
            if version:
                version = version[0]
                return version
    elif os.path.exists(changelog):
        f = open(changelog)
        head = f.read().strip().split('\n')[0]
        f.close()
        rev = re.compile('\d+\.\d+\.(\d+)').findall(head)
        if rev:
            return '3.0.%s' % rev[0]
    return '3.0.x'


setup(
    name="ox",
    version=get_version(),
    description="python-ox - the web in a dict",
    author="0x2620",
    author_email="0x2620@0x2620.org",
    url="https://code.0x2620.org/0x2620/python-ox",
    license="GPLv3",
    packages=['ox', 'ox.torrent', 'ox.web'],
    install_requires=['chardet', 'requests'],
    keywords=[
    ],
    classifiers=[
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)

